# Generated from XQueryParser.g4 by ANTLR 4.9.2
from antlr4 import *
if __name__ is not None and "." in __name__:
    from .XQueryParser import XQueryParser
else:
    from XQueryParser import XQueryParser

# This class defines a complete listener for a parse tree produced by XQueryParser.
class XQueryParserListener(ParseTreeListener):

    # Enter a parse tree produced by XQueryParser#module.
    def enterModule(self, ctx:XQueryParser.ModuleContext):
        pass

    # Exit a parse tree produced by XQueryParser#module.
    def exitModule(self, ctx:XQueryParser.ModuleContext):
        pass


    # Enter a parse tree produced by XQueryParser#xqDocComment.
    def enterXqDocComment(self, ctx:XQueryParser.XqDocCommentContext):
        pass

    # Exit a parse tree produced by XQueryParser#xqDocComment.
    def exitXqDocComment(self, ctx:XQueryParser.XqDocCommentContext):
        pass


    # Enter a parse tree produced by XQueryParser#versionDecl.
    def enterVersionDecl(self, ctx:XQueryParser.VersionDeclContext):
        pass

    # Exit a parse tree produced by XQueryParser#versionDecl.
    def exitVersionDecl(self, ctx:XQueryParser.VersionDeclContext):
        pass


    # Enter a parse tree produced by XQueryParser#mainModule.
    def enterMainModule(self, ctx:XQueryParser.MainModuleContext):
        pass

    # Exit a parse tree produced by XQueryParser#mainModule.
    def exitMainModule(self, ctx:XQueryParser.MainModuleContext):
        pass


    # Enter a parse tree produced by XQueryParser#queryBody.
    def enterQueryBody(self, ctx:XQueryParser.QueryBodyContext):
        pass

    # Exit a parse tree produced by XQueryParser#queryBody.
    def exitQueryBody(self, ctx:XQueryParser.QueryBodyContext):
        pass


    # Enter a parse tree produced by XQueryParser#libraryModule.
    def enterLibraryModule(self, ctx:XQueryParser.LibraryModuleContext):
        pass

    # Exit a parse tree produced by XQueryParser#libraryModule.
    def exitLibraryModule(self, ctx:XQueryParser.LibraryModuleContext):
        pass


    # Enter a parse tree produced by XQueryParser#moduleDecl.
    def enterModuleDecl(self, ctx:XQueryParser.ModuleDeclContext):
        pass

    # Exit a parse tree produced by XQueryParser#moduleDecl.
    def exitModuleDecl(self, ctx:XQueryParser.ModuleDeclContext):
        pass


    # Enter a parse tree produced by XQueryParser#prolog.
    def enterProlog(self, ctx:XQueryParser.PrologContext):
        pass

    # Exit a parse tree produced by XQueryParser#prolog.
    def exitProlog(self, ctx:XQueryParser.PrologContext):
        pass


    # Enter a parse tree produced by XQueryParser#defaultNamespaceDecl.
    def enterDefaultNamespaceDecl(self, ctx:XQueryParser.DefaultNamespaceDeclContext):
        pass

    # Exit a parse tree produced by XQueryParser#defaultNamespaceDecl.
    def exitDefaultNamespaceDecl(self, ctx:XQueryParser.DefaultNamespaceDeclContext):
        pass


    # Enter a parse tree produced by XQueryParser#setter.
    def enterSetter(self, ctx:XQueryParser.SetterContext):
        pass

    # Exit a parse tree produced by XQueryParser#setter.
    def exitSetter(self, ctx:XQueryParser.SetterContext):
        pass


    # Enter a parse tree produced by XQueryParser#boundarySpaceDecl.
    def enterBoundarySpaceDecl(self, ctx:XQueryParser.BoundarySpaceDeclContext):
        pass

    # Exit a parse tree produced by XQueryParser#boundarySpaceDecl.
    def exitBoundarySpaceDecl(self, ctx:XQueryParser.BoundarySpaceDeclContext):
        pass


    # Enter a parse tree produced by XQueryParser#defaultCollationDecl.
    def enterDefaultCollationDecl(self, ctx:XQueryParser.DefaultCollationDeclContext):
        pass

    # Exit a parse tree produced by XQueryParser#defaultCollationDecl.
    def exitDefaultCollationDecl(self, ctx:XQueryParser.DefaultCollationDeclContext):
        pass


    # Enter a parse tree produced by XQueryParser#baseURIDecl.
    def enterBaseURIDecl(self, ctx:XQueryParser.BaseURIDeclContext):
        pass

    # Exit a parse tree produced by XQueryParser#baseURIDecl.
    def exitBaseURIDecl(self, ctx:XQueryParser.BaseURIDeclContext):
        pass


    # Enter a parse tree produced by XQueryParser#constructionDecl.
    def enterConstructionDecl(self, ctx:XQueryParser.ConstructionDeclContext):
        pass

    # Exit a parse tree produced by XQueryParser#constructionDecl.
    def exitConstructionDecl(self, ctx:XQueryParser.ConstructionDeclContext):
        pass


    # Enter a parse tree produced by XQueryParser#orderingModeDecl.
    def enterOrderingModeDecl(self, ctx:XQueryParser.OrderingModeDeclContext):
        pass

    # Exit a parse tree produced by XQueryParser#orderingModeDecl.
    def exitOrderingModeDecl(self, ctx:XQueryParser.OrderingModeDeclContext):
        pass


    # Enter a parse tree produced by XQueryParser#emptyOrderDecl.
    def enterEmptyOrderDecl(self, ctx:XQueryParser.EmptyOrderDeclContext):
        pass

    # Exit a parse tree produced by XQueryParser#emptyOrderDecl.
    def exitEmptyOrderDecl(self, ctx:XQueryParser.EmptyOrderDeclContext):
        pass


    # Enter a parse tree produced by XQueryParser#copyNamespacesDecl.
    def enterCopyNamespacesDecl(self, ctx:XQueryParser.CopyNamespacesDeclContext):
        pass

    # Exit a parse tree produced by XQueryParser#copyNamespacesDecl.
    def exitCopyNamespacesDecl(self, ctx:XQueryParser.CopyNamespacesDeclContext):
        pass


    # Enter a parse tree produced by XQueryParser#preserveMode.
    def enterPreserveMode(self, ctx:XQueryParser.PreserveModeContext):
        pass

    # Exit a parse tree produced by XQueryParser#preserveMode.
    def exitPreserveMode(self, ctx:XQueryParser.PreserveModeContext):
        pass


    # Enter a parse tree produced by XQueryParser#inheritMode.
    def enterInheritMode(self, ctx:XQueryParser.InheritModeContext):
        pass

    # Exit a parse tree produced by XQueryParser#inheritMode.
    def exitInheritMode(self, ctx:XQueryParser.InheritModeContext):
        pass


    # Enter a parse tree produced by XQueryParser#decimalFormatDecl.
    def enterDecimalFormatDecl(self, ctx:XQueryParser.DecimalFormatDeclContext):
        pass

    # Exit a parse tree produced by XQueryParser#decimalFormatDecl.
    def exitDecimalFormatDecl(self, ctx:XQueryParser.DecimalFormatDeclContext):
        pass


    # Enter a parse tree produced by XQueryParser#schemaImport.
    def enterSchemaImport(self, ctx:XQueryParser.SchemaImportContext):
        pass

    # Exit a parse tree produced by XQueryParser#schemaImport.
    def exitSchemaImport(self, ctx:XQueryParser.SchemaImportContext):
        pass


    # Enter a parse tree produced by XQueryParser#schemaPrefix.
    def enterSchemaPrefix(self, ctx:XQueryParser.SchemaPrefixContext):
        pass

    # Exit a parse tree produced by XQueryParser#schemaPrefix.
    def exitSchemaPrefix(self, ctx:XQueryParser.SchemaPrefixContext):
        pass


    # Enter a parse tree produced by XQueryParser#moduleImport.
    def enterModuleImport(self, ctx:XQueryParser.ModuleImportContext):
        pass

    # Exit a parse tree produced by XQueryParser#moduleImport.
    def exitModuleImport(self, ctx:XQueryParser.ModuleImportContext):
        pass


    # Enter a parse tree produced by XQueryParser#namespaceDecl.
    def enterNamespaceDecl(self, ctx:XQueryParser.NamespaceDeclContext):
        pass

    # Exit a parse tree produced by XQueryParser#namespaceDecl.
    def exitNamespaceDecl(self, ctx:XQueryParser.NamespaceDeclContext):
        pass


    # Enter a parse tree produced by XQueryParser#varDecl.
    def enterVarDecl(self, ctx:XQueryParser.VarDeclContext):
        pass

    # Exit a parse tree produced by XQueryParser#varDecl.
    def exitVarDecl(self, ctx:XQueryParser.VarDeclContext):
        pass


    # Enter a parse tree produced by XQueryParser#varValue.
    def enterVarValue(self, ctx:XQueryParser.VarValueContext):
        pass

    # Exit a parse tree produced by XQueryParser#varValue.
    def exitVarValue(self, ctx:XQueryParser.VarValueContext):
        pass


    # Enter a parse tree produced by XQueryParser#varDefaultValue.
    def enterVarDefaultValue(self, ctx:XQueryParser.VarDefaultValueContext):
        pass

    # Exit a parse tree produced by XQueryParser#varDefaultValue.
    def exitVarDefaultValue(self, ctx:XQueryParser.VarDefaultValueContext):
        pass


    # Enter a parse tree produced by XQueryParser#contextItemDecl.
    def enterContextItemDecl(self, ctx:XQueryParser.ContextItemDeclContext):
        pass

    # Exit a parse tree produced by XQueryParser#contextItemDecl.
    def exitContextItemDecl(self, ctx:XQueryParser.ContextItemDeclContext):
        pass


    # Enter a parse tree produced by XQueryParser#functionDecl.
    def enterFunctionDecl(self, ctx:XQueryParser.FunctionDeclContext):
        pass

    # Exit a parse tree produced by XQueryParser#functionDecl.
    def exitFunctionDecl(self, ctx:XQueryParser.FunctionDeclContext):
        pass


    # Enter a parse tree produced by XQueryParser#functionParams.
    def enterFunctionParams(self, ctx:XQueryParser.FunctionParamsContext):
        pass

    # Exit a parse tree produced by XQueryParser#functionParams.
    def exitFunctionParams(self, ctx:XQueryParser.FunctionParamsContext):
        pass


    # Enter a parse tree produced by XQueryParser#functionParam.
    def enterFunctionParam(self, ctx:XQueryParser.FunctionParamContext):
        pass

    # Exit a parse tree produced by XQueryParser#functionParam.
    def exitFunctionParam(self, ctx:XQueryParser.FunctionParamContext):
        pass


    # Enter a parse tree produced by XQueryParser#annotations.
    def enterAnnotations(self, ctx:XQueryParser.AnnotationsContext):
        pass

    # Exit a parse tree produced by XQueryParser#annotations.
    def exitAnnotations(self, ctx:XQueryParser.AnnotationsContext):
        pass


    # Enter a parse tree produced by XQueryParser#annotation.
    def enterAnnotation(self, ctx:XQueryParser.AnnotationContext):
        pass

    # Exit a parse tree produced by XQueryParser#annotation.
    def exitAnnotation(self, ctx:XQueryParser.AnnotationContext):
        pass


    # Enter a parse tree produced by XQueryParser#annotList.
    def enterAnnotList(self, ctx:XQueryParser.AnnotListContext):
        pass

    # Exit a parse tree produced by XQueryParser#annotList.
    def exitAnnotList(self, ctx:XQueryParser.AnnotListContext):
        pass


    # Enter a parse tree produced by XQueryParser#annotationParam.
    def enterAnnotationParam(self, ctx:XQueryParser.AnnotationParamContext):
        pass

    # Exit a parse tree produced by XQueryParser#annotationParam.
    def exitAnnotationParam(self, ctx:XQueryParser.AnnotationParamContext):
        pass


    # Enter a parse tree produced by XQueryParser#functionReturn.
    def enterFunctionReturn(self, ctx:XQueryParser.FunctionReturnContext):
        pass

    # Exit a parse tree produced by XQueryParser#functionReturn.
    def exitFunctionReturn(self, ctx:XQueryParser.FunctionReturnContext):
        pass


    # Enter a parse tree produced by XQueryParser#optionDecl.
    def enterOptionDecl(self, ctx:XQueryParser.OptionDeclContext):
        pass

    # Exit a parse tree produced by XQueryParser#optionDecl.
    def exitOptionDecl(self, ctx:XQueryParser.OptionDeclContext):
        pass


    # Enter a parse tree produced by XQueryParser#expr.
    def enterExpr(self, ctx:XQueryParser.ExprContext):
        pass

    # Exit a parse tree produced by XQueryParser#expr.
    def exitExpr(self, ctx:XQueryParser.ExprContext):
        pass


    # Enter a parse tree produced by XQueryParser#exprSingle.
    def enterExprSingle(self, ctx:XQueryParser.ExprSingleContext):
        pass

    # Exit a parse tree produced by XQueryParser#exprSingle.
    def exitExprSingle(self, ctx:XQueryParser.ExprSingleContext):
        pass


    # Enter a parse tree produced by XQueryParser#flworExpr.
    def enterFlworExpr(self, ctx:XQueryParser.FlworExprContext):
        pass

    # Exit a parse tree produced by XQueryParser#flworExpr.
    def exitFlworExpr(self, ctx:XQueryParser.FlworExprContext):
        pass


    # Enter a parse tree produced by XQueryParser#initialClause.
    def enterInitialClause(self, ctx:XQueryParser.InitialClauseContext):
        pass

    # Exit a parse tree produced by XQueryParser#initialClause.
    def exitInitialClause(self, ctx:XQueryParser.InitialClauseContext):
        pass


    # Enter a parse tree produced by XQueryParser#intermediateClause.
    def enterIntermediateClause(self, ctx:XQueryParser.IntermediateClauseContext):
        pass

    # Exit a parse tree produced by XQueryParser#intermediateClause.
    def exitIntermediateClause(self, ctx:XQueryParser.IntermediateClauseContext):
        pass


    # Enter a parse tree produced by XQueryParser#forClause.
    def enterForClause(self, ctx:XQueryParser.ForClauseContext):
        pass

    # Exit a parse tree produced by XQueryParser#forClause.
    def exitForClause(self, ctx:XQueryParser.ForClauseContext):
        pass


    # Enter a parse tree produced by XQueryParser#forBinding.
    def enterForBinding(self, ctx:XQueryParser.ForBindingContext):
        pass

    # Exit a parse tree produced by XQueryParser#forBinding.
    def exitForBinding(self, ctx:XQueryParser.ForBindingContext):
        pass


    # Enter a parse tree produced by XQueryParser#allowingEmpty.
    def enterAllowingEmpty(self, ctx:XQueryParser.AllowingEmptyContext):
        pass

    # Exit a parse tree produced by XQueryParser#allowingEmpty.
    def exitAllowingEmpty(self, ctx:XQueryParser.AllowingEmptyContext):
        pass


    # Enter a parse tree produced by XQueryParser#positionalVar.
    def enterPositionalVar(self, ctx:XQueryParser.PositionalVarContext):
        pass

    # Exit a parse tree produced by XQueryParser#positionalVar.
    def exitPositionalVar(self, ctx:XQueryParser.PositionalVarContext):
        pass


    # Enter a parse tree produced by XQueryParser#letClause.
    def enterLetClause(self, ctx:XQueryParser.LetClauseContext):
        pass

    # Exit a parse tree produced by XQueryParser#letClause.
    def exitLetClause(self, ctx:XQueryParser.LetClauseContext):
        pass


    # Enter a parse tree produced by XQueryParser#letBinding.
    def enterLetBinding(self, ctx:XQueryParser.LetBindingContext):
        pass

    # Exit a parse tree produced by XQueryParser#letBinding.
    def exitLetBinding(self, ctx:XQueryParser.LetBindingContext):
        pass


    # Enter a parse tree produced by XQueryParser#windowClause.
    def enterWindowClause(self, ctx:XQueryParser.WindowClauseContext):
        pass

    # Exit a parse tree produced by XQueryParser#windowClause.
    def exitWindowClause(self, ctx:XQueryParser.WindowClauseContext):
        pass


    # Enter a parse tree produced by XQueryParser#tumblingWindowClause.
    def enterTumblingWindowClause(self, ctx:XQueryParser.TumblingWindowClauseContext):
        pass

    # Exit a parse tree produced by XQueryParser#tumblingWindowClause.
    def exitTumblingWindowClause(self, ctx:XQueryParser.TumblingWindowClauseContext):
        pass


    # Enter a parse tree produced by XQueryParser#slidingWindowClause.
    def enterSlidingWindowClause(self, ctx:XQueryParser.SlidingWindowClauseContext):
        pass

    # Exit a parse tree produced by XQueryParser#slidingWindowClause.
    def exitSlidingWindowClause(self, ctx:XQueryParser.SlidingWindowClauseContext):
        pass


    # Enter a parse tree produced by XQueryParser#windowStartCondition.
    def enterWindowStartCondition(self, ctx:XQueryParser.WindowStartConditionContext):
        pass

    # Exit a parse tree produced by XQueryParser#windowStartCondition.
    def exitWindowStartCondition(self, ctx:XQueryParser.WindowStartConditionContext):
        pass


    # Enter a parse tree produced by XQueryParser#windowEndCondition.
    def enterWindowEndCondition(self, ctx:XQueryParser.WindowEndConditionContext):
        pass

    # Exit a parse tree produced by XQueryParser#windowEndCondition.
    def exitWindowEndCondition(self, ctx:XQueryParser.WindowEndConditionContext):
        pass


    # Enter a parse tree produced by XQueryParser#windowVars.
    def enterWindowVars(self, ctx:XQueryParser.WindowVarsContext):
        pass

    # Exit a parse tree produced by XQueryParser#windowVars.
    def exitWindowVars(self, ctx:XQueryParser.WindowVarsContext):
        pass


    # Enter a parse tree produced by XQueryParser#countClause.
    def enterCountClause(self, ctx:XQueryParser.CountClauseContext):
        pass

    # Exit a parse tree produced by XQueryParser#countClause.
    def exitCountClause(self, ctx:XQueryParser.CountClauseContext):
        pass


    # Enter a parse tree produced by XQueryParser#whereClause.
    def enterWhereClause(self, ctx:XQueryParser.WhereClauseContext):
        pass

    # Exit a parse tree produced by XQueryParser#whereClause.
    def exitWhereClause(self, ctx:XQueryParser.WhereClauseContext):
        pass


    # Enter a parse tree produced by XQueryParser#groupByClause.
    def enterGroupByClause(self, ctx:XQueryParser.GroupByClauseContext):
        pass

    # Exit a parse tree produced by XQueryParser#groupByClause.
    def exitGroupByClause(self, ctx:XQueryParser.GroupByClauseContext):
        pass


    # Enter a parse tree produced by XQueryParser#groupingSpecList.
    def enterGroupingSpecList(self, ctx:XQueryParser.GroupingSpecListContext):
        pass

    # Exit a parse tree produced by XQueryParser#groupingSpecList.
    def exitGroupingSpecList(self, ctx:XQueryParser.GroupingSpecListContext):
        pass


    # Enter a parse tree produced by XQueryParser#groupingSpec.
    def enterGroupingSpec(self, ctx:XQueryParser.GroupingSpecContext):
        pass

    # Exit a parse tree produced by XQueryParser#groupingSpec.
    def exitGroupingSpec(self, ctx:XQueryParser.GroupingSpecContext):
        pass


    # Enter a parse tree produced by XQueryParser#orderByClause.
    def enterOrderByClause(self, ctx:XQueryParser.OrderByClauseContext):
        pass

    # Exit a parse tree produced by XQueryParser#orderByClause.
    def exitOrderByClause(self, ctx:XQueryParser.OrderByClauseContext):
        pass


    # Enter a parse tree produced by XQueryParser#orderSpec.
    def enterOrderSpec(self, ctx:XQueryParser.OrderSpecContext):
        pass

    # Exit a parse tree produced by XQueryParser#orderSpec.
    def exitOrderSpec(self, ctx:XQueryParser.OrderSpecContext):
        pass


    # Enter a parse tree produced by XQueryParser#returnClause.
    def enterReturnClause(self, ctx:XQueryParser.ReturnClauseContext):
        pass

    # Exit a parse tree produced by XQueryParser#returnClause.
    def exitReturnClause(self, ctx:XQueryParser.ReturnClauseContext):
        pass


    # Enter a parse tree produced by XQueryParser#quantifiedExpr.
    def enterQuantifiedExpr(self, ctx:XQueryParser.QuantifiedExprContext):
        pass

    # Exit a parse tree produced by XQueryParser#quantifiedExpr.
    def exitQuantifiedExpr(self, ctx:XQueryParser.QuantifiedExprContext):
        pass


    # Enter a parse tree produced by XQueryParser#quantifiedVar.
    def enterQuantifiedVar(self, ctx:XQueryParser.QuantifiedVarContext):
        pass

    # Exit a parse tree produced by XQueryParser#quantifiedVar.
    def exitQuantifiedVar(self, ctx:XQueryParser.QuantifiedVarContext):
        pass


    # Enter a parse tree produced by XQueryParser#switchExpr.
    def enterSwitchExpr(self, ctx:XQueryParser.SwitchExprContext):
        pass

    # Exit a parse tree produced by XQueryParser#switchExpr.
    def exitSwitchExpr(self, ctx:XQueryParser.SwitchExprContext):
        pass


    # Enter a parse tree produced by XQueryParser#switchCaseClause.
    def enterSwitchCaseClause(self, ctx:XQueryParser.SwitchCaseClauseContext):
        pass

    # Exit a parse tree produced by XQueryParser#switchCaseClause.
    def exitSwitchCaseClause(self, ctx:XQueryParser.SwitchCaseClauseContext):
        pass


    # Enter a parse tree produced by XQueryParser#switchCaseOperand.
    def enterSwitchCaseOperand(self, ctx:XQueryParser.SwitchCaseOperandContext):
        pass

    # Exit a parse tree produced by XQueryParser#switchCaseOperand.
    def exitSwitchCaseOperand(self, ctx:XQueryParser.SwitchCaseOperandContext):
        pass


    # Enter a parse tree produced by XQueryParser#typeswitchExpr.
    def enterTypeswitchExpr(self, ctx:XQueryParser.TypeswitchExprContext):
        pass

    # Exit a parse tree produced by XQueryParser#typeswitchExpr.
    def exitTypeswitchExpr(self, ctx:XQueryParser.TypeswitchExprContext):
        pass


    # Enter a parse tree produced by XQueryParser#caseClause.
    def enterCaseClause(self, ctx:XQueryParser.CaseClauseContext):
        pass

    # Exit a parse tree produced by XQueryParser#caseClause.
    def exitCaseClause(self, ctx:XQueryParser.CaseClauseContext):
        pass


    # Enter a parse tree produced by XQueryParser#sequenceUnionType.
    def enterSequenceUnionType(self, ctx:XQueryParser.SequenceUnionTypeContext):
        pass

    # Exit a parse tree produced by XQueryParser#sequenceUnionType.
    def exitSequenceUnionType(self, ctx:XQueryParser.SequenceUnionTypeContext):
        pass


    # Enter a parse tree produced by XQueryParser#ifExpr.
    def enterIfExpr(self, ctx:XQueryParser.IfExprContext):
        pass

    # Exit a parse tree produced by XQueryParser#ifExpr.
    def exitIfExpr(self, ctx:XQueryParser.IfExprContext):
        pass


    # Enter a parse tree produced by XQueryParser#tryCatchExpr.
    def enterTryCatchExpr(self, ctx:XQueryParser.TryCatchExprContext):
        pass

    # Exit a parse tree produced by XQueryParser#tryCatchExpr.
    def exitTryCatchExpr(self, ctx:XQueryParser.TryCatchExprContext):
        pass


    # Enter a parse tree produced by XQueryParser#tryClause.
    def enterTryClause(self, ctx:XQueryParser.TryClauseContext):
        pass

    # Exit a parse tree produced by XQueryParser#tryClause.
    def exitTryClause(self, ctx:XQueryParser.TryClauseContext):
        pass


    # Enter a parse tree produced by XQueryParser#enclosedTryTargetExpression.
    def enterEnclosedTryTargetExpression(self, ctx:XQueryParser.EnclosedTryTargetExpressionContext):
        pass

    # Exit a parse tree produced by XQueryParser#enclosedTryTargetExpression.
    def exitEnclosedTryTargetExpression(self, ctx:XQueryParser.EnclosedTryTargetExpressionContext):
        pass


    # Enter a parse tree produced by XQueryParser#catchClause.
    def enterCatchClause(self, ctx:XQueryParser.CatchClauseContext):
        pass

    # Exit a parse tree produced by XQueryParser#catchClause.
    def exitCatchClause(self, ctx:XQueryParser.CatchClauseContext):
        pass


    # Enter a parse tree produced by XQueryParser#enclosedExpression.
    def enterEnclosedExpression(self, ctx:XQueryParser.EnclosedExpressionContext):
        pass

    # Exit a parse tree produced by XQueryParser#enclosedExpression.
    def exitEnclosedExpression(self, ctx:XQueryParser.EnclosedExpressionContext):
        pass


    # Enter a parse tree produced by XQueryParser#catchErrorList.
    def enterCatchErrorList(self, ctx:XQueryParser.CatchErrorListContext):
        pass

    # Exit a parse tree produced by XQueryParser#catchErrorList.
    def exitCatchErrorList(self, ctx:XQueryParser.CatchErrorListContext):
        pass


    # Enter a parse tree produced by XQueryParser#existUpdateExpr.
    def enterExistUpdateExpr(self, ctx:XQueryParser.ExistUpdateExprContext):
        pass

    # Exit a parse tree produced by XQueryParser#existUpdateExpr.
    def exitExistUpdateExpr(self, ctx:XQueryParser.ExistUpdateExprContext):
        pass


    # Enter a parse tree produced by XQueryParser#existReplaceExpr.
    def enterExistReplaceExpr(self, ctx:XQueryParser.ExistReplaceExprContext):
        pass

    # Exit a parse tree produced by XQueryParser#existReplaceExpr.
    def exitExistReplaceExpr(self, ctx:XQueryParser.ExistReplaceExprContext):
        pass


    # Enter a parse tree produced by XQueryParser#existValueExpr.
    def enterExistValueExpr(self, ctx:XQueryParser.ExistValueExprContext):
        pass

    # Exit a parse tree produced by XQueryParser#existValueExpr.
    def exitExistValueExpr(self, ctx:XQueryParser.ExistValueExprContext):
        pass


    # Enter a parse tree produced by XQueryParser#existInsertExpr.
    def enterExistInsertExpr(self, ctx:XQueryParser.ExistInsertExprContext):
        pass

    # Exit a parse tree produced by XQueryParser#existInsertExpr.
    def exitExistInsertExpr(self, ctx:XQueryParser.ExistInsertExprContext):
        pass


    # Enter a parse tree produced by XQueryParser#existDeleteExpr.
    def enterExistDeleteExpr(self, ctx:XQueryParser.ExistDeleteExprContext):
        pass

    # Exit a parse tree produced by XQueryParser#existDeleteExpr.
    def exitExistDeleteExpr(self, ctx:XQueryParser.ExistDeleteExprContext):
        pass


    # Enter a parse tree produced by XQueryParser#existRenameExpr.
    def enterExistRenameExpr(self, ctx:XQueryParser.ExistRenameExprContext):
        pass

    # Exit a parse tree produced by XQueryParser#existRenameExpr.
    def exitExistRenameExpr(self, ctx:XQueryParser.ExistRenameExprContext):
        pass


    # Enter a parse tree produced by XQueryParser#orExpr.
    def enterOrExpr(self, ctx:XQueryParser.OrExprContext):
        pass

    # Exit a parse tree produced by XQueryParser#orExpr.
    def exitOrExpr(self, ctx:XQueryParser.OrExprContext):
        pass


    # Enter a parse tree produced by XQueryParser#andExpr.
    def enterAndExpr(self, ctx:XQueryParser.AndExprContext):
        pass

    # Exit a parse tree produced by XQueryParser#andExpr.
    def exitAndExpr(self, ctx:XQueryParser.AndExprContext):
        pass


    # Enter a parse tree produced by XQueryParser#comparisonExpr.
    def enterComparisonExpr(self, ctx:XQueryParser.ComparisonExprContext):
        pass

    # Exit a parse tree produced by XQueryParser#comparisonExpr.
    def exitComparisonExpr(self, ctx:XQueryParser.ComparisonExprContext):
        pass


    # Enter a parse tree produced by XQueryParser#stringConcatExpr.
    def enterStringConcatExpr(self, ctx:XQueryParser.StringConcatExprContext):
        pass

    # Exit a parse tree produced by XQueryParser#stringConcatExpr.
    def exitStringConcatExpr(self, ctx:XQueryParser.StringConcatExprContext):
        pass


    # Enter a parse tree produced by XQueryParser#rangeExpr.
    def enterRangeExpr(self, ctx:XQueryParser.RangeExprContext):
        pass

    # Exit a parse tree produced by XQueryParser#rangeExpr.
    def exitRangeExpr(self, ctx:XQueryParser.RangeExprContext):
        pass


    # Enter a parse tree produced by XQueryParser#additiveExpr.
    def enterAdditiveExpr(self, ctx:XQueryParser.AdditiveExprContext):
        pass

    # Exit a parse tree produced by XQueryParser#additiveExpr.
    def exitAdditiveExpr(self, ctx:XQueryParser.AdditiveExprContext):
        pass


    # Enter a parse tree produced by XQueryParser#multiplicativeExpr.
    def enterMultiplicativeExpr(self, ctx:XQueryParser.MultiplicativeExprContext):
        pass

    # Exit a parse tree produced by XQueryParser#multiplicativeExpr.
    def exitMultiplicativeExpr(self, ctx:XQueryParser.MultiplicativeExprContext):
        pass


    # Enter a parse tree produced by XQueryParser#unionExpr.
    def enterUnionExpr(self, ctx:XQueryParser.UnionExprContext):
        pass

    # Exit a parse tree produced by XQueryParser#unionExpr.
    def exitUnionExpr(self, ctx:XQueryParser.UnionExprContext):
        pass


    # Enter a parse tree produced by XQueryParser#intersectExceptExpr.
    def enterIntersectExceptExpr(self, ctx:XQueryParser.IntersectExceptExprContext):
        pass

    # Exit a parse tree produced by XQueryParser#intersectExceptExpr.
    def exitIntersectExceptExpr(self, ctx:XQueryParser.IntersectExceptExprContext):
        pass


    # Enter a parse tree produced by XQueryParser#instanceOfExpr.
    def enterInstanceOfExpr(self, ctx:XQueryParser.InstanceOfExprContext):
        pass

    # Exit a parse tree produced by XQueryParser#instanceOfExpr.
    def exitInstanceOfExpr(self, ctx:XQueryParser.InstanceOfExprContext):
        pass


    # Enter a parse tree produced by XQueryParser#treatExpr.
    def enterTreatExpr(self, ctx:XQueryParser.TreatExprContext):
        pass

    # Exit a parse tree produced by XQueryParser#treatExpr.
    def exitTreatExpr(self, ctx:XQueryParser.TreatExprContext):
        pass


    # Enter a parse tree produced by XQueryParser#castableExpr.
    def enterCastableExpr(self, ctx:XQueryParser.CastableExprContext):
        pass

    # Exit a parse tree produced by XQueryParser#castableExpr.
    def exitCastableExpr(self, ctx:XQueryParser.CastableExprContext):
        pass


    # Enter a parse tree produced by XQueryParser#castExpr.
    def enterCastExpr(self, ctx:XQueryParser.CastExprContext):
        pass

    # Exit a parse tree produced by XQueryParser#castExpr.
    def exitCastExpr(self, ctx:XQueryParser.CastExprContext):
        pass


    # Enter a parse tree produced by XQueryParser#arrowExpr.
    def enterArrowExpr(self, ctx:XQueryParser.ArrowExprContext):
        pass

    # Exit a parse tree produced by XQueryParser#arrowExpr.
    def exitArrowExpr(self, ctx:XQueryParser.ArrowExprContext):
        pass


    # Enter a parse tree produced by XQueryParser#unaryExpression.
    def enterUnaryExpression(self, ctx:XQueryParser.UnaryExpressionContext):
        pass

    # Exit a parse tree produced by XQueryParser#unaryExpression.
    def exitUnaryExpression(self, ctx:XQueryParser.UnaryExpressionContext):
        pass


    # Enter a parse tree produced by XQueryParser#valueExpr.
    def enterValueExpr(self, ctx:XQueryParser.ValueExprContext):
        pass

    # Exit a parse tree produced by XQueryParser#valueExpr.
    def exitValueExpr(self, ctx:XQueryParser.ValueExprContext):
        pass


    # Enter a parse tree produced by XQueryParser#generalComp.
    def enterGeneralComp(self, ctx:XQueryParser.GeneralCompContext):
        pass

    # Exit a parse tree produced by XQueryParser#generalComp.
    def exitGeneralComp(self, ctx:XQueryParser.GeneralCompContext):
        pass


    # Enter a parse tree produced by XQueryParser#valueComp.
    def enterValueComp(self, ctx:XQueryParser.ValueCompContext):
        pass

    # Exit a parse tree produced by XQueryParser#valueComp.
    def exitValueComp(self, ctx:XQueryParser.ValueCompContext):
        pass


    # Enter a parse tree produced by XQueryParser#nodeComp.
    def enterNodeComp(self, ctx:XQueryParser.NodeCompContext):
        pass

    # Exit a parse tree produced by XQueryParser#nodeComp.
    def exitNodeComp(self, ctx:XQueryParser.NodeCompContext):
        pass


    # Enter a parse tree produced by XQueryParser#validateExpr.
    def enterValidateExpr(self, ctx:XQueryParser.ValidateExprContext):
        pass

    # Exit a parse tree produced by XQueryParser#validateExpr.
    def exitValidateExpr(self, ctx:XQueryParser.ValidateExprContext):
        pass


    # Enter a parse tree produced by XQueryParser#validationMode.
    def enterValidationMode(self, ctx:XQueryParser.ValidationModeContext):
        pass

    # Exit a parse tree produced by XQueryParser#validationMode.
    def exitValidationMode(self, ctx:XQueryParser.ValidationModeContext):
        pass


    # Enter a parse tree produced by XQueryParser#extensionExpr.
    def enterExtensionExpr(self, ctx:XQueryParser.ExtensionExprContext):
        pass

    # Exit a parse tree produced by XQueryParser#extensionExpr.
    def exitExtensionExpr(self, ctx:XQueryParser.ExtensionExprContext):
        pass


    # Enter a parse tree produced by XQueryParser#simpleMapExpr.
    def enterSimpleMapExpr(self, ctx:XQueryParser.SimpleMapExprContext):
        pass

    # Exit a parse tree produced by XQueryParser#simpleMapExpr.
    def exitSimpleMapExpr(self, ctx:XQueryParser.SimpleMapExprContext):
        pass


    # Enter a parse tree produced by XQueryParser#pathExpr.
    def enterPathExpr(self, ctx:XQueryParser.PathExprContext):
        pass

    # Exit a parse tree produced by XQueryParser#pathExpr.
    def exitPathExpr(self, ctx:XQueryParser.PathExprContext):
        pass


    # Enter a parse tree produced by XQueryParser#relativePathExpr.
    def enterRelativePathExpr(self, ctx:XQueryParser.RelativePathExprContext):
        pass

    # Exit a parse tree produced by XQueryParser#relativePathExpr.
    def exitRelativePathExpr(self, ctx:XQueryParser.RelativePathExprContext):
        pass


    # Enter a parse tree produced by XQueryParser#stepExpr.
    def enterStepExpr(self, ctx:XQueryParser.StepExprContext):
        pass

    # Exit a parse tree produced by XQueryParser#stepExpr.
    def exitStepExpr(self, ctx:XQueryParser.StepExprContext):
        pass


    # Enter a parse tree produced by XQueryParser#axisStep.
    def enterAxisStep(self, ctx:XQueryParser.AxisStepContext):
        pass

    # Exit a parse tree produced by XQueryParser#axisStep.
    def exitAxisStep(self, ctx:XQueryParser.AxisStepContext):
        pass


    # Enter a parse tree produced by XQueryParser#forwardStep.
    def enterForwardStep(self, ctx:XQueryParser.ForwardStepContext):
        pass

    # Exit a parse tree produced by XQueryParser#forwardStep.
    def exitForwardStep(self, ctx:XQueryParser.ForwardStepContext):
        pass


    # Enter a parse tree produced by XQueryParser#forwardAxis.
    def enterForwardAxis(self, ctx:XQueryParser.ForwardAxisContext):
        pass

    # Exit a parse tree produced by XQueryParser#forwardAxis.
    def exitForwardAxis(self, ctx:XQueryParser.ForwardAxisContext):
        pass


    # Enter a parse tree produced by XQueryParser#abbrevForwardStep.
    def enterAbbrevForwardStep(self, ctx:XQueryParser.AbbrevForwardStepContext):
        pass

    # Exit a parse tree produced by XQueryParser#abbrevForwardStep.
    def exitAbbrevForwardStep(self, ctx:XQueryParser.AbbrevForwardStepContext):
        pass


    # Enter a parse tree produced by XQueryParser#reverseStep.
    def enterReverseStep(self, ctx:XQueryParser.ReverseStepContext):
        pass

    # Exit a parse tree produced by XQueryParser#reverseStep.
    def exitReverseStep(self, ctx:XQueryParser.ReverseStepContext):
        pass


    # Enter a parse tree produced by XQueryParser#reverseAxis.
    def enterReverseAxis(self, ctx:XQueryParser.ReverseAxisContext):
        pass

    # Exit a parse tree produced by XQueryParser#reverseAxis.
    def exitReverseAxis(self, ctx:XQueryParser.ReverseAxisContext):
        pass


    # Enter a parse tree produced by XQueryParser#abbrevReverseStep.
    def enterAbbrevReverseStep(self, ctx:XQueryParser.AbbrevReverseStepContext):
        pass

    # Exit a parse tree produced by XQueryParser#abbrevReverseStep.
    def exitAbbrevReverseStep(self, ctx:XQueryParser.AbbrevReverseStepContext):
        pass


    # Enter a parse tree produced by XQueryParser#nodeTest.
    def enterNodeTest(self, ctx:XQueryParser.NodeTestContext):
        pass

    # Exit a parse tree produced by XQueryParser#nodeTest.
    def exitNodeTest(self, ctx:XQueryParser.NodeTestContext):
        pass


    # Enter a parse tree produced by XQueryParser#nameTest.
    def enterNameTest(self, ctx:XQueryParser.NameTestContext):
        pass

    # Exit a parse tree produced by XQueryParser#nameTest.
    def exitNameTest(self, ctx:XQueryParser.NameTestContext):
        pass


    # Enter a parse tree produced by XQueryParser#allNames.
    def enterAllNames(self, ctx:XQueryParser.AllNamesContext):
        pass

    # Exit a parse tree produced by XQueryParser#allNames.
    def exitAllNames(self, ctx:XQueryParser.AllNamesContext):
        pass


    # Enter a parse tree produced by XQueryParser#allWithNS.
    def enterAllWithNS(self, ctx:XQueryParser.AllWithNSContext):
        pass

    # Exit a parse tree produced by XQueryParser#allWithNS.
    def exitAllWithNS(self, ctx:XQueryParser.AllWithNSContext):
        pass


    # Enter a parse tree produced by XQueryParser#allWithLocal.
    def enterAllWithLocal(self, ctx:XQueryParser.AllWithLocalContext):
        pass

    # Exit a parse tree produced by XQueryParser#allWithLocal.
    def exitAllWithLocal(self, ctx:XQueryParser.AllWithLocalContext):
        pass


    # Enter a parse tree produced by XQueryParser#postfixExpr.
    def enterPostfixExpr(self, ctx:XQueryParser.PostfixExprContext):
        pass

    # Exit a parse tree produced by XQueryParser#postfixExpr.
    def exitPostfixExpr(self, ctx:XQueryParser.PostfixExprContext):
        pass


    # Enter a parse tree produced by XQueryParser#argumentList.
    def enterArgumentList(self, ctx:XQueryParser.ArgumentListContext):
        pass

    # Exit a parse tree produced by XQueryParser#argumentList.
    def exitArgumentList(self, ctx:XQueryParser.ArgumentListContext):
        pass


    # Enter a parse tree produced by XQueryParser#predicateList.
    def enterPredicateList(self, ctx:XQueryParser.PredicateListContext):
        pass

    # Exit a parse tree produced by XQueryParser#predicateList.
    def exitPredicateList(self, ctx:XQueryParser.PredicateListContext):
        pass


    # Enter a parse tree produced by XQueryParser#predicate.
    def enterPredicate(self, ctx:XQueryParser.PredicateContext):
        pass

    # Exit a parse tree produced by XQueryParser#predicate.
    def exitPredicate(self, ctx:XQueryParser.PredicateContext):
        pass


    # Enter a parse tree produced by XQueryParser#lookup.
    def enterLookup(self, ctx:XQueryParser.LookupContext):
        pass

    # Exit a parse tree produced by XQueryParser#lookup.
    def exitLookup(self, ctx:XQueryParser.LookupContext):
        pass


    # Enter a parse tree produced by XQueryParser#keySpecifier.
    def enterKeySpecifier(self, ctx:XQueryParser.KeySpecifierContext):
        pass

    # Exit a parse tree produced by XQueryParser#keySpecifier.
    def exitKeySpecifier(self, ctx:XQueryParser.KeySpecifierContext):
        pass


    # Enter a parse tree produced by XQueryParser#arrowFunctionSpecifier.
    def enterArrowFunctionSpecifier(self, ctx:XQueryParser.ArrowFunctionSpecifierContext):
        pass

    # Exit a parse tree produced by XQueryParser#arrowFunctionSpecifier.
    def exitArrowFunctionSpecifier(self, ctx:XQueryParser.ArrowFunctionSpecifierContext):
        pass


    # Enter a parse tree produced by XQueryParser#primaryExpr.
    def enterPrimaryExpr(self, ctx:XQueryParser.PrimaryExprContext):
        pass

    # Exit a parse tree produced by XQueryParser#primaryExpr.
    def exitPrimaryExpr(self, ctx:XQueryParser.PrimaryExprContext):
        pass


    # Enter a parse tree produced by XQueryParser#literal.
    def enterLiteral(self, ctx:XQueryParser.LiteralContext):
        pass

    # Exit a parse tree produced by XQueryParser#literal.
    def exitLiteral(self, ctx:XQueryParser.LiteralContext):
        pass


    # Enter a parse tree produced by XQueryParser#numericLiteral.
    def enterNumericLiteral(self, ctx:XQueryParser.NumericLiteralContext):
        pass

    # Exit a parse tree produced by XQueryParser#numericLiteral.
    def exitNumericLiteral(self, ctx:XQueryParser.NumericLiteralContext):
        pass


    # Enter a parse tree produced by XQueryParser#varRef.
    def enterVarRef(self, ctx:XQueryParser.VarRefContext):
        pass

    # Exit a parse tree produced by XQueryParser#varRef.
    def exitVarRef(self, ctx:XQueryParser.VarRefContext):
        pass


    # Enter a parse tree produced by XQueryParser#varName.
    def enterVarName(self, ctx:XQueryParser.VarNameContext):
        pass

    # Exit a parse tree produced by XQueryParser#varName.
    def exitVarName(self, ctx:XQueryParser.VarNameContext):
        pass


    # Enter a parse tree produced by XQueryParser#parenthesizedExpr.
    def enterParenthesizedExpr(self, ctx:XQueryParser.ParenthesizedExprContext):
        pass

    # Exit a parse tree produced by XQueryParser#parenthesizedExpr.
    def exitParenthesizedExpr(self, ctx:XQueryParser.ParenthesizedExprContext):
        pass


    # Enter a parse tree produced by XQueryParser#contextItemExpr.
    def enterContextItemExpr(self, ctx:XQueryParser.ContextItemExprContext):
        pass

    # Exit a parse tree produced by XQueryParser#contextItemExpr.
    def exitContextItemExpr(self, ctx:XQueryParser.ContextItemExprContext):
        pass


    # Enter a parse tree produced by XQueryParser#orderedExpr.
    def enterOrderedExpr(self, ctx:XQueryParser.OrderedExprContext):
        pass

    # Exit a parse tree produced by XQueryParser#orderedExpr.
    def exitOrderedExpr(self, ctx:XQueryParser.OrderedExprContext):
        pass


    # Enter a parse tree produced by XQueryParser#unorderedExpr.
    def enterUnorderedExpr(self, ctx:XQueryParser.UnorderedExprContext):
        pass

    # Exit a parse tree produced by XQueryParser#unorderedExpr.
    def exitUnorderedExpr(self, ctx:XQueryParser.UnorderedExprContext):
        pass


    # Enter a parse tree produced by XQueryParser#functionCall.
    def enterFunctionCall(self, ctx:XQueryParser.FunctionCallContext):
        pass

    # Exit a parse tree produced by XQueryParser#functionCall.
    def exitFunctionCall(self, ctx:XQueryParser.FunctionCallContext):
        pass


    # Enter a parse tree produced by XQueryParser#argument.
    def enterArgument(self, ctx:XQueryParser.ArgumentContext):
        pass

    # Exit a parse tree produced by XQueryParser#argument.
    def exitArgument(self, ctx:XQueryParser.ArgumentContext):
        pass


    # Enter a parse tree produced by XQueryParser#nodeConstructor.
    def enterNodeConstructor(self, ctx:XQueryParser.NodeConstructorContext):
        pass

    # Exit a parse tree produced by XQueryParser#nodeConstructor.
    def exitNodeConstructor(self, ctx:XQueryParser.NodeConstructorContext):
        pass


    # Enter a parse tree produced by XQueryParser#directConstructor.
    def enterDirectConstructor(self, ctx:XQueryParser.DirectConstructorContext):
        pass

    # Exit a parse tree produced by XQueryParser#directConstructor.
    def exitDirectConstructor(self, ctx:XQueryParser.DirectConstructorContext):
        pass


    # Enter a parse tree produced by XQueryParser#dirElemConstructorOpenClose.
    def enterDirElemConstructorOpenClose(self, ctx:XQueryParser.DirElemConstructorOpenCloseContext):
        pass

    # Exit a parse tree produced by XQueryParser#dirElemConstructorOpenClose.
    def exitDirElemConstructorOpenClose(self, ctx:XQueryParser.DirElemConstructorOpenCloseContext):
        pass


    # Enter a parse tree produced by XQueryParser#dirElemConstructorSingleTag.
    def enterDirElemConstructorSingleTag(self, ctx:XQueryParser.DirElemConstructorSingleTagContext):
        pass

    # Exit a parse tree produced by XQueryParser#dirElemConstructorSingleTag.
    def exitDirElemConstructorSingleTag(self, ctx:XQueryParser.DirElemConstructorSingleTagContext):
        pass


    # Enter a parse tree produced by XQueryParser#dirAttributeList.
    def enterDirAttributeList(self, ctx:XQueryParser.DirAttributeListContext):
        pass

    # Exit a parse tree produced by XQueryParser#dirAttributeList.
    def exitDirAttributeList(self, ctx:XQueryParser.DirAttributeListContext):
        pass


    # Enter a parse tree produced by XQueryParser#dirAttributeValueApos.
    def enterDirAttributeValueApos(self, ctx:XQueryParser.DirAttributeValueAposContext):
        pass

    # Exit a parse tree produced by XQueryParser#dirAttributeValueApos.
    def exitDirAttributeValueApos(self, ctx:XQueryParser.DirAttributeValueAposContext):
        pass


    # Enter a parse tree produced by XQueryParser#dirAttributeValueQuot.
    def enterDirAttributeValueQuot(self, ctx:XQueryParser.DirAttributeValueQuotContext):
        pass

    # Exit a parse tree produced by XQueryParser#dirAttributeValueQuot.
    def exitDirAttributeValueQuot(self, ctx:XQueryParser.DirAttributeValueQuotContext):
        pass


    # Enter a parse tree produced by XQueryParser#dirAttributeValue.
    def enterDirAttributeValue(self, ctx:XQueryParser.DirAttributeValueContext):
        pass

    # Exit a parse tree produced by XQueryParser#dirAttributeValue.
    def exitDirAttributeValue(self, ctx:XQueryParser.DirAttributeValueContext):
        pass


    # Enter a parse tree produced by XQueryParser#dirAttributeContentQuot.
    def enterDirAttributeContentQuot(self, ctx:XQueryParser.DirAttributeContentQuotContext):
        pass

    # Exit a parse tree produced by XQueryParser#dirAttributeContentQuot.
    def exitDirAttributeContentQuot(self, ctx:XQueryParser.DirAttributeContentQuotContext):
        pass


    # Enter a parse tree produced by XQueryParser#dirAttributeContentApos.
    def enterDirAttributeContentApos(self, ctx:XQueryParser.DirAttributeContentAposContext):
        pass

    # Exit a parse tree produced by XQueryParser#dirAttributeContentApos.
    def exitDirAttributeContentApos(self, ctx:XQueryParser.DirAttributeContentAposContext):
        pass


    # Enter a parse tree produced by XQueryParser#dirElemContent.
    def enterDirElemContent(self, ctx:XQueryParser.DirElemContentContext):
        pass

    # Exit a parse tree produced by XQueryParser#dirElemContent.
    def exitDirElemContent(self, ctx:XQueryParser.DirElemContentContext):
        pass


    # Enter a parse tree produced by XQueryParser#commonContent.
    def enterCommonContent(self, ctx:XQueryParser.CommonContentContext):
        pass

    # Exit a parse tree produced by XQueryParser#commonContent.
    def exitCommonContent(self, ctx:XQueryParser.CommonContentContext):
        pass


    # Enter a parse tree produced by XQueryParser#computedConstructor.
    def enterComputedConstructor(self, ctx:XQueryParser.ComputedConstructorContext):
        pass

    # Exit a parse tree produced by XQueryParser#computedConstructor.
    def exitComputedConstructor(self, ctx:XQueryParser.ComputedConstructorContext):
        pass


    # Enter a parse tree produced by XQueryParser#compMLJSONConstructor.
    def enterCompMLJSONConstructor(self, ctx:XQueryParser.CompMLJSONConstructorContext):
        pass

    # Exit a parse tree produced by XQueryParser#compMLJSONConstructor.
    def exitCompMLJSONConstructor(self, ctx:XQueryParser.CompMLJSONConstructorContext):
        pass


    # Enter a parse tree produced by XQueryParser#compMLJSONArrayConstructor.
    def enterCompMLJSONArrayConstructor(self, ctx:XQueryParser.CompMLJSONArrayConstructorContext):
        pass

    # Exit a parse tree produced by XQueryParser#compMLJSONArrayConstructor.
    def exitCompMLJSONArrayConstructor(self, ctx:XQueryParser.CompMLJSONArrayConstructorContext):
        pass


    # Enter a parse tree produced by XQueryParser#compMLJSONObjectConstructor.
    def enterCompMLJSONObjectConstructor(self, ctx:XQueryParser.CompMLJSONObjectConstructorContext):
        pass

    # Exit a parse tree produced by XQueryParser#compMLJSONObjectConstructor.
    def exitCompMLJSONObjectConstructor(self, ctx:XQueryParser.CompMLJSONObjectConstructorContext):
        pass


    # Enter a parse tree produced by XQueryParser#compMLJSONNumberConstructor.
    def enterCompMLJSONNumberConstructor(self, ctx:XQueryParser.CompMLJSONNumberConstructorContext):
        pass

    # Exit a parse tree produced by XQueryParser#compMLJSONNumberConstructor.
    def exitCompMLJSONNumberConstructor(self, ctx:XQueryParser.CompMLJSONNumberConstructorContext):
        pass


    # Enter a parse tree produced by XQueryParser#compMLJSONBooleanConstructor.
    def enterCompMLJSONBooleanConstructor(self, ctx:XQueryParser.CompMLJSONBooleanConstructorContext):
        pass

    # Exit a parse tree produced by XQueryParser#compMLJSONBooleanConstructor.
    def exitCompMLJSONBooleanConstructor(self, ctx:XQueryParser.CompMLJSONBooleanConstructorContext):
        pass


    # Enter a parse tree produced by XQueryParser#compMLJSONNullConstructor.
    def enterCompMLJSONNullConstructor(self, ctx:XQueryParser.CompMLJSONNullConstructorContext):
        pass

    # Exit a parse tree produced by XQueryParser#compMLJSONNullConstructor.
    def exitCompMLJSONNullConstructor(self, ctx:XQueryParser.CompMLJSONNullConstructorContext):
        pass


    # Enter a parse tree produced by XQueryParser#compBinaryConstructor.
    def enterCompBinaryConstructor(self, ctx:XQueryParser.CompBinaryConstructorContext):
        pass

    # Exit a parse tree produced by XQueryParser#compBinaryConstructor.
    def exitCompBinaryConstructor(self, ctx:XQueryParser.CompBinaryConstructorContext):
        pass


    # Enter a parse tree produced by XQueryParser#compDocConstructor.
    def enterCompDocConstructor(self, ctx:XQueryParser.CompDocConstructorContext):
        pass

    # Exit a parse tree produced by XQueryParser#compDocConstructor.
    def exitCompDocConstructor(self, ctx:XQueryParser.CompDocConstructorContext):
        pass


    # Enter a parse tree produced by XQueryParser#compElemConstructor.
    def enterCompElemConstructor(self, ctx:XQueryParser.CompElemConstructorContext):
        pass

    # Exit a parse tree produced by XQueryParser#compElemConstructor.
    def exitCompElemConstructor(self, ctx:XQueryParser.CompElemConstructorContext):
        pass


    # Enter a parse tree produced by XQueryParser#enclosedContentExpr.
    def enterEnclosedContentExpr(self, ctx:XQueryParser.EnclosedContentExprContext):
        pass

    # Exit a parse tree produced by XQueryParser#enclosedContentExpr.
    def exitEnclosedContentExpr(self, ctx:XQueryParser.EnclosedContentExprContext):
        pass


    # Enter a parse tree produced by XQueryParser#compAttrConstructor.
    def enterCompAttrConstructor(self, ctx:XQueryParser.CompAttrConstructorContext):
        pass

    # Exit a parse tree produced by XQueryParser#compAttrConstructor.
    def exitCompAttrConstructor(self, ctx:XQueryParser.CompAttrConstructorContext):
        pass


    # Enter a parse tree produced by XQueryParser#compNamespaceConstructor.
    def enterCompNamespaceConstructor(self, ctx:XQueryParser.CompNamespaceConstructorContext):
        pass

    # Exit a parse tree produced by XQueryParser#compNamespaceConstructor.
    def exitCompNamespaceConstructor(self, ctx:XQueryParser.CompNamespaceConstructorContext):
        pass


    # Enter a parse tree produced by XQueryParser#prefix.
    def enterPrefix(self, ctx:XQueryParser.PrefixContext):
        pass

    # Exit a parse tree produced by XQueryParser#prefix.
    def exitPrefix(self, ctx:XQueryParser.PrefixContext):
        pass


    # Enter a parse tree produced by XQueryParser#enclosedPrefixExpr.
    def enterEnclosedPrefixExpr(self, ctx:XQueryParser.EnclosedPrefixExprContext):
        pass

    # Exit a parse tree produced by XQueryParser#enclosedPrefixExpr.
    def exitEnclosedPrefixExpr(self, ctx:XQueryParser.EnclosedPrefixExprContext):
        pass


    # Enter a parse tree produced by XQueryParser#enclosedURIExpr.
    def enterEnclosedURIExpr(self, ctx:XQueryParser.EnclosedURIExprContext):
        pass

    # Exit a parse tree produced by XQueryParser#enclosedURIExpr.
    def exitEnclosedURIExpr(self, ctx:XQueryParser.EnclosedURIExprContext):
        pass


    # Enter a parse tree produced by XQueryParser#compTextConstructor.
    def enterCompTextConstructor(self, ctx:XQueryParser.CompTextConstructorContext):
        pass

    # Exit a parse tree produced by XQueryParser#compTextConstructor.
    def exitCompTextConstructor(self, ctx:XQueryParser.CompTextConstructorContext):
        pass


    # Enter a parse tree produced by XQueryParser#compCommentConstructor.
    def enterCompCommentConstructor(self, ctx:XQueryParser.CompCommentConstructorContext):
        pass

    # Exit a parse tree produced by XQueryParser#compCommentConstructor.
    def exitCompCommentConstructor(self, ctx:XQueryParser.CompCommentConstructorContext):
        pass


    # Enter a parse tree produced by XQueryParser#compPIConstructor.
    def enterCompPIConstructor(self, ctx:XQueryParser.CompPIConstructorContext):
        pass

    # Exit a parse tree produced by XQueryParser#compPIConstructor.
    def exitCompPIConstructor(self, ctx:XQueryParser.CompPIConstructorContext):
        pass


    # Enter a parse tree produced by XQueryParser#functionItemExpr.
    def enterFunctionItemExpr(self, ctx:XQueryParser.FunctionItemExprContext):
        pass

    # Exit a parse tree produced by XQueryParser#functionItemExpr.
    def exitFunctionItemExpr(self, ctx:XQueryParser.FunctionItemExprContext):
        pass


    # Enter a parse tree produced by XQueryParser#namedFunctionRef.
    def enterNamedFunctionRef(self, ctx:XQueryParser.NamedFunctionRefContext):
        pass

    # Exit a parse tree produced by XQueryParser#namedFunctionRef.
    def exitNamedFunctionRef(self, ctx:XQueryParser.NamedFunctionRefContext):
        pass


    # Enter a parse tree produced by XQueryParser#inlineFunctionRef.
    def enterInlineFunctionRef(self, ctx:XQueryParser.InlineFunctionRefContext):
        pass

    # Exit a parse tree produced by XQueryParser#inlineFunctionRef.
    def exitInlineFunctionRef(self, ctx:XQueryParser.InlineFunctionRefContext):
        pass


    # Enter a parse tree produced by XQueryParser#functionBody.
    def enterFunctionBody(self, ctx:XQueryParser.FunctionBodyContext):
        pass

    # Exit a parse tree produced by XQueryParser#functionBody.
    def exitFunctionBody(self, ctx:XQueryParser.FunctionBodyContext):
        pass


    # Enter a parse tree produced by XQueryParser#mapConstructor.
    def enterMapConstructor(self, ctx:XQueryParser.MapConstructorContext):
        pass

    # Exit a parse tree produced by XQueryParser#mapConstructor.
    def exitMapConstructor(self, ctx:XQueryParser.MapConstructorContext):
        pass


    # Enter a parse tree produced by XQueryParser#mapConstructorEntry.
    def enterMapConstructorEntry(self, ctx:XQueryParser.MapConstructorEntryContext):
        pass

    # Exit a parse tree produced by XQueryParser#mapConstructorEntry.
    def exitMapConstructorEntry(self, ctx:XQueryParser.MapConstructorEntryContext):
        pass


    # Enter a parse tree produced by XQueryParser#arrayConstructor.
    def enterArrayConstructor(self, ctx:XQueryParser.ArrayConstructorContext):
        pass

    # Exit a parse tree produced by XQueryParser#arrayConstructor.
    def exitArrayConstructor(self, ctx:XQueryParser.ArrayConstructorContext):
        pass


    # Enter a parse tree produced by XQueryParser#squareArrayConstructor.
    def enterSquareArrayConstructor(self, ctx:XQueryParser.SquareArrayConstructorContext):
        pass

    # Exit a parse tree produced by XQueryParser#squareArrayConstructor.
    def exitSquareArrayConstructor(self, ctx:XQueryParser.SquareArrayConstructorContext):
        pass


    # Enter a parse tree produced by XQueryParser#curlyArrayConstructor.
    def enterCurlyArrayConstructor(self, ctx:XQueryParser.CurlyArrayConstructorContext):
        pass

    # Exit a parse tree produced by XQueryParser#curlyArrayConstructor.
    def exitCurlyArrayConstructor(self, ctx:XQueryParser.CurlyArrayConstructorContext):
        pass


    # Enter a parse tree produced by XQueryParser#stringConstructor.
    def enterStringConstructor(self, ctx:XQueryParser.StringConstructorContext):
        pass

    # Exit a parse tree produced by XQueryParser#stringConstructor.
    def exitStringConstructor(self, ctx:XQueryParser.StringConstructorContext):
        pass


    # Enter a parse tree produced by XQueryParser#stringConstructorContent.
    def enterStringConstructorContent(self, ctx:XQueryParser.StringConstructorContentContext):
        pass

    # Exit a parse tree produced by XQueryParser#stringConstructorContent.
    def exitStringConstructorContent(self, ctx:XQueryParser.StringConstructorContentContext):
        pass


    # Enter a parse tree produced by XQueryParser#charNoGrave.
    def enterCharNoGrave(self, ctx:XQueryParser.CharNoGraveContext):
        pass

    # Exit a parse tree produced by XQueryParser#charNoGrave.
    def exitCharNoGrave(self, ctx:XQueryParser.CharNoGraveContext):
        pass


    # Enter a parse tree produced by XQueryParser#charNoLBrace.
    def enterCharNoLBrace(self, ctx:XQueryParser.CharNoLBraceContext):
        pass

    # Exit a parse tree produced by XQueryParser#charNoLBrace.
    def exitCharNoLBrace(self, ctx:XQueryParser.CharNoLBraceContext):
        pass


    # Enter a parse tree produced by XQueryParser#charNoRBrack.
    def enterCharNoRBrack(self, ctx:XQueryParser.CharNoRBrackContext):
        pass

    # Exit a parse tree produced by XQueryParser#charNoRBrack.
    def exitCharNoRBrack(self, ctx:XQueryParser.CharNoRBrackContext):
        pass


    # Enter a parse tree produced by XQueryParser#stringConstructorChars.
    def enterStringConstructorChars(self, ctx:XQueryParser.StringConstructorCharsContext):
        pass

    # Exit a parse tree produced by XQueryParser#stringConstructorChars.
    def exitStringConstructorChars(self, ctx:XQueryParser.StringConstructorCharsContext):
        pass


    # Enter a parse tree produced by XQueryParser#stringConstructorInterpolation.
    def enterStringConstructorInterpolation(self, ctx:XQueryParser.StringConstructorInterpolationContext):
        pass

    # Exit a parse tree produced by XQueryParser#stringConstructorInterpolation.
    def exitStringConstructorInterpolation(self, ctx:XQueryParser.StringConstructorInterpolationContext):
        pass


    # Enter a parse tree produced by XQueryParser#unaryLookup.
    def enterUnaryLookup(self, ctx:XQueryParser.UnaryLookupContext):
        pass

    # Exit a parse tree produced by XQueryParser#unaryLookup.
    def exitUnaryLookup(self, ctx:XQueryParser.UnaryLookupContext):
        pass


    # Enter a parse tree produced by XQueryParser#singleType.
    def enterSingleType(self, ctx:XQueryParser.SingleTypeContext):
        pass

    # Exit a parse tree produced by XQueryParser#singleType.
    def exitSingleType(self, ctx:XQueryParser.SingleTypeContext):
        pass


    # Enter a parse tree produced by XQueryParser#typeDeclaration.
    def enterTypeDeclaration(self, ctx:XQueryParser.TypeDeclarationContext):
        pass

    # Exit a parse tree produced by XQueryParser#typeDeclaration.
    def exitTypeDeclaration(self, ctx:XQueryParser.TypeDeclarationContext):
        pass


    # Enter a parse tree produced by XQueryParser#sequenceType.
    def enterSequenceType(self, ctx:XQueryParser.SequenceTypeContext):
        pass

    # Exit a parse tree produced by XQueryParser#sequenceType.
    def exitSequenceType(self, ctx:XQueryParser.SequenceTypeContext):
        pass


    # Enter a parse tree produced by XQueryParser#itemType.
    def enterItemType(self, ctx:XQueryParser.ItemTypeContext):
        pass

    # Exit a parse tree produced by XQueryParser#itemType.
    def exitItemType(self, ctx:XQueryParser.ItemTypeContext):
        pass


    # Enter a parse tree produced by XQueryParser#atomicOrUnionType.
    def enterAtomicOrUnionType(self, ctx:XQueryParser.AtomicOrUnionTypeContext):
        pass

    # Exit a parse tree produced by XQueryParser#atomicOrUnionType.
    def exitAtomicOrUnionType(self, ctx:XQueryParser.AtomicOrUnionTypeContext):
        pass


    # Enter a parse tree produced by XQueryParser#kindTest.
    def enterKindTest(self, ctx:XQueryParser.KindTestContext):
        pass

    # Exit a parse tree produced by XQueryParser#kindTest.
    def exitKindTest(self, ctx:XQueryParser.KindTestContext):
        pass


    # Enter a parse tree produced by XQueryParser#anyKindTest.
    def enterAnyKindTest(self, ctx:XQueryParser.AnyKindTestContext):
        pass

    # Exit a parse tree produced by XQueryParser#anyKindTest.
    def exitAnyKindTest(self, ctx:XQueryParser.AnyKindTestContext):
        pass


    # Enter a parse tree produced by XQueryParser#binaryNodeTest.
    def enterBinaryNodeTest(self, ctx:XQueryParser.BinaryNodeTestContext):
        pass

    # Exit a parse tree produced by XQueryParser#binaryNodeTest.
    def exitBinaryNodeTest(self, ctx:XQueryParser.BinaryNodeTestContext):
        pass


    # Enter a parse tree produced by XQueryParser#documentTest.
    def enterDocumentTest(self, ctx:XQueryParser.DocumentTestContext):
        pass

    # Exit a parse tree produced by XQueryParser#documentTest.
    def exitDocumentTest(self, ctx:XQueryParser.DocumentTestContext):
        pass


    # Enter a parse tree produced by XQueryParser#textTest.
    def enterTextTest(self, ctx:XQueryParser.TextTestContext):
        pass

    # Exit a parse tree produced by XQueryParser#textTest.
    def exitTextTest(self, ctx:XQueryParser.TextTestContext):
        pass


    # Enter a parse tree produced by XQueryParser#commentTest.
    def enterCommentTest(self, ctx:XQueryParser.CommentTestContext):
        pass

    # Exit a parse tree produced by XQueryParser#commentTest.
    def exitCommentTest(self, ctx:XQueryParser.CommentTestContext):
        pass


    # Enter a parse tree produced by XQueryParser#namespaceNodeTest.
    def enterNamespaceNodeTest(self, ctx:XQueryParser.NamespaceNodeTestContext):
        pass

    # Exit a parse tree produced by XQueryParser#namespaceNodeTest.
    def exitNamespaceNodeTest(self, ctx:XQueryParser.NamespaceNodeTestContext):
        pass


    # Enter a parse tree produced by XQueryParser#piTest.
    def enterPiTest(self, ctx:XQueryParser.PiTestContext):
        pass

    # Exit a parse tree produced by XQueryParser#piTest.
    def exitPiTest(self, ctx:XQueryParser.PiTestContext):
        pass


    # Enter a parse tree produced by XQueryParser#attributeTest.
    def enterAttributeTest(self, ctx:XQueryParser.AttributeTestContext):
        pass

    # Exit a parse tree produced by XQueryParser#attributeTest.
    def exitAttributeTest(self, ctx:XQueryParser.AttributeTestContext):
        pass


    # Enter a parse tree produced by XQueryParser#attributeNameOrWildcard.
    def enterAttributeNameOrWildcard(self, ctx:XQueryParser.AttributeNameOrWildcardContext):
        pass

    # Exit a parse tree produced by XQueryParser#attributeNameOrWildcard.
    def exitAttributeNameOrWildcard(self, ctx:XQueryParser.AttributeNameOrWildcardContext):
        pass


    # Enter a parse tree produced by XQueryParser#schemaAttributeTest.
    def enterSchemaAttributeTest(self, ctx:XQueryParser.SchemaAttributeTestContext):
        pass

    # Exit a parse tree produced by XQueryParser#schemaAttributeTest.
    def exitSchemaAttributeTest(self, ctx:XQueryParser.SchemaAttributeTestContext):
        pass


    # Enter a parse tree produced by XQueryParser#elementTest.
    def enterElementTest(self, ctx:XQueryParser.ElementTestContext):
        pass

    # Exit a parse tree produced by XQueryParser#elementTest.
    def exitElementTest(self, ctx:XQueryParser.ElementTestContext):
        pass


    # Enter a parse tree produced by XQueryParser#elementNameOrWildcard.
    def enterElementNameOrWildcard(self, ctx:XQueryParser.ElementNameOrWildcardContext):
        pass

    # Exit a parse tree produced by XQueryParser#elementNameOrWildcard.
    def exitElementNameOrWildcard(self, ctx:XQueryParser.ElementNameOrWildcardContext):
        pass


    # Enter a parse tree produced by XQueryParser#schemaElementTest.
    def enterSchemaElementTest(self, ctx:XQueryParser.SchemaElementTestContext):
        pass

    # Exit a parse tree produced by XQueryParser#schemaElementTest.
    def exitSchemaElementTest(self, ctx:XQueryParser.SchemaElementTestContext):
        pass


    # Enter a parse tree produced by XQueryParser#elementDeclaration.
    def enterElementDeclaration(self, ctx:XQueryParser.ElementDeclarationContext):
        pass

    # Exit a parse tree produced by XQueryParser#elementDeclaration.
    def exitElementDeclaration(self, ctx:XQueryParser.ElementDeclarationContext):
        pass


    # Enter a parse tree produced by XQueryParser#attributeName.
    def enterAttributeName(self, ctx:XQueryParser.AttributeNameContext):
        pass

    # Exit a parse tree produced by XQueryParser#attributeName.
    def exitAttributeName(self, ctx:XQueryParser.AttributeNameContext):
        pass


    # Enter a parse tree produced by XQueryParser#elementName.
    def enterElementName(self, ctx:XQueryParser.ElementNameContext):
        pass

    # Exit a parse tree produced by XQueryParser#elementName.
    def exitElementName(self, ctx:XQueryParser.ElementNameContext):
        pass


    # Enter a parse tree produced by XQueryParser#simpleTypeName.
    def enterSimpleTypeName(self, ctx:XQueryParser.SimpleTypeNameContext):
        pass

    # Exit a parse tree produced by XQueryParser#simpleTypeName.
    def exitSimpleTypeName(self, ctx:XQueryParser.SimpleTypeNameContext):
        pass


    # Enter a parse tree produced by XQueryParser#typeName.
    def enterTypeName(self, ctx:XQueryParser.TypeNameContext):
        pass

    # Exit a parse tree produced by XQueryParser#typeName.
    def exitTypeName(self, ctx:XQueryParser.TypeNameContext):
        pass


    # Enter a parse tree produced by XQueryParser#functionTest.
    def enterFunctionTest(self, ctx:XQueryParser.FunctionTestContext):
        pass

    # Exit a parse tree produced by XQueryParser#functionTest.
    def exitFunctionTest(self, ctx:XQueryParser.FunctionTestContext):
        pass


    # Enter a parse tree produced by XQueryParser#anyFunctionTest.
    def enterAnyFunctionTest(self, ctx:XQueryParser.AnyFunctionTestContext):
        pass

    # Exit a parse tree produced by XQueryParser#anyFunctionTest.
    def exitAnyFunctionTest(self, ctx:XQueryParser.AnyFunctionTestContext):
        pass


    # Enter a parse tree produced by XQueryParser#typedFunctionTest.
    def enterTypedFunctionTest(self, ctx:XQueryParser.TypedFunctionTestContext):
        pass

    # Exit a parse tree produced by XQueryParser#typedFunctionTest.
    def exitTypedFunctionTest(self, ctx:XQueryParser.TypedFunctionTestContext):
        pass


    # Enter a parse tree produced by XQueryParser#mapTest.
    def enterMapTest(self, ctx:XQueryParser.MapTestContext):
        pass

    # Exit a parse tree produced by XQueryParser#mapTest.
    def exitMapTest(self, ctx:XQueryParser.MapTestContext):
        pass


    # Enter a parse tree produced by XQueryParser#anyMapTest.
    def enterAnyMapTest(self, ctx:XQueryParser.AnyMapTestContext):
        pass

    # Exit a parse tree produced by XQueryParser#anyMapTest.
    def exitAnyMapTest(self, ctx:XQueryParser.AnyMapTestContext):
        pass


    # Enter a parse tree produced by XQueryParser#typedMapTest.
    def enterTypedMapTest(self, ctx:XQueryParser.TypedMapTestContext):
        pass

    # Exit a parse tree produced by XQueryParser#typedMapTest.
    def exitTypedMapTest(self, ctx:XQueryParser.TypedMapTestContext):
        pass


    # Enter a parse tree produced by XQueryParser#arrayTest.
    def enterArrayTest(self, ctx:XQueryParser.ArrayTestContext):
        pass

    # Exit a parse tree produced by XQueryParser#arrayTest.
    def exitArrayTest(self, ctx:XQueryParser.ArrayTestContext):
        pass


    # Enter a parse tree produced by XQueryParser#anyArrayTest.
    def enterAnyArrayTest(self, ctx:XQueryParser.AnyArrayTestContext):
        pass

    # Exit a parse tree produced by XQueryParser#anyArrayTest.
    def exitAnyArrayTest(self, ctx:XQueryParser.AnyArrayTestContext):
        pass


    # Enter a parse tree produced by XQueryParser#typedArrayTest.
    def enterTypedArrayTest(self, ctx:XQueryParser.TypedArrayTestContext):
        pass

    # Exit a parse tree produced by XQueryParser#typedArrayTest.
    def exitTypedArrayTest(self, ctx:XQueryParser.TypedArrayTestContext):
        pass


    # Enter a parse tree produced by XQueryParser#parenthesizedItemTest.
    def enterParenthesizedItemTest(self, ctx:XQueryParser.ParenthesizedItemTestContext):
        pass

    # Exit a parse tree produced by XQueryParser#parenthesizedItemTest.
    def exitParenthesizedItemTest(self, ctx:XQueryParser.ParenthesizedItemTestContext):
        pass


    # Enter a parse tree produced by XQueryParser#attributeDeclaration.
    def enterAttributeDeclaration(self, ctx:XQueryParser.AttributeDeclarationContext):
        pass

    # Exit a parse tree produced by XQueryParser#attributeDeclaration.
    def exitAttributeDeclaration(self, ctx:XQueryParser.AttributeDeclarationContext):
        pass


    # Enter a parse tree produced by XQueryParser#mlNodeTest.
    def enterMlNodeTest(self, ctx:XQueryParser.MlNodeTestContext):
        pass

    # Exit a parse tree produced by XQueryParser#mlNodeTest.
    def exitMlNodeTest(self, ctx:XQueryParser.MlNodeTestContext):
        pass


    # Enter a parse tree produced by XQueryParser#mlArrayNodeTest.
    def enterMlArrayNodeTest(self, ctx:XQueryParser.MlArrayNodeTestContext):
        pass

    # Exit a parse tree produced by XQueryParser#mlArrayNodeTest.
    def exitMlArrayNodeTest(self, ctx:XQueryParser.MlArrayNodeTestContext):
        pass


    # Enter a parse tree produced by XQueryParser#mlObjectNodeTest.
    def enterMlObjectNodeTest(self, ctx:XQueryParser.MlObjectNodeTestContext):
        pass

    # Exit a parse tree produced by XQueryParser#mlObjectNodeTest.
    def exitMlObjectNodeTest(self, ctx:XQueryParser.MlObjectNodeTestContext):
        pass


    # Enter a parse tree produced by XQueryParser#mlNumberNodeTest.
    def enterMlNumberNodeTest(self, ctx:XQueryParser.MlNumberNodeTestContext):
        pass

    # Exit a parse tree produced by XQueryParser#mlNumberNodeTest.
    def exitMlNumberNodeTest(self, ctx:XQueryParser.MlNumberNodeTestContext):
        pass


    # Enter a parse tree produced by XQueryParser#mlBooleanNodeTest.
    def enterMlBooleanNodeTest(self, ctx:XQueryParser.MlBooleanNodeTestContext):
        pass

    # Exit a parse tree produced by XQueryParser#mlBooleanNodeTest.
    def exitMlBooleanNodeTest(self, ctx:XQueryParser.MlBooleanNodeTestContext):
        pass


    # Enter a parse tree produced by XQueryParser#mlNullNodeTest.
    def enterMlNullNodeTest(self, ctx:XQueryParser.MlNullNodeTestContext):
        pass

    # Exit a parse tree produced by XQueryParser#mlNullNodeTest.
    def exitMlNullNodeTest(self, ctx:XQueryParser.MlNullNodeTestContext):
        pass


    # Enter a parse tree produced by XQueryParser#eqName.
    def enterEqName(self, ctx:XQueryParser.EqNameContext):
        pass

    # Exit a parse tree produced by XQueryParser#eqName.
    def exitEqName(self, ctx:XQueryParser.EqNameContext):
        pass


    # Enter a parse tree produced by XQueryParser#qName.
    def enterQName(self, ctx:XQueryParser.QNameContext):
        pass

    # Exit a parse tree produced by XQueryParser#qName.
    def exitQName(self, ctx:XQueryParser.QNameContext):
        pass


    # Enter a parse tree produced by XQueryParser#ncName.
    def enterNcName(self, ctx:XQueryParser.NcNameContext):
        pass

    # Exit a parse tree produced by XQueryParser#ncName.
    def exitNcName(self, ctx:XQueryParser.NcNameContext):
        pass


    # Enter a parse tree produced by XQueryParser#functionName.
    def enterFunctionName(self, ctx:XQueryParser.FunctionNameContext):
        pass

    # Exit a parse tree produced by XQueryParser#functionName.
    def exitFunctionName(self, ctx:XQueryParser.FunctionNameContext):
        pass


    # Enter a parse tree produced by XQueryParser#keyword.
    def enterKeyword(self, ctx:XQueryParser.KeywordContext):
        pass

    # Exit a parse tree produced by XQueryParser#keyword.
    def exitKeyword(self, ctx:XQueryParser.KeywordContext):
        pass


    # Enter a parse tree produced by XQueryParser#keywordNotOKForFunction.
    def enterKeywordNotOKForFunction(self, ctx:XQueryParser.KeywordNotOKForFunctionContext):
        pass

    # Exit a parse tree produced by XQueryParser#keywordNotOKForFunction.
    def exitKeywordNotOKForFunction(self, ctx:XQueryParser.KeywordNotOKForFunctionContext):
        pass


    # Enter a parse tree produced by XQueryParser#keywordOKForFunction.
    def enterKeywordOKForFunction(self, ctx:XQueryParser.KeywordOKForFunctionContext):
        pass

    # Exit a parse tree produced by XQueryParser#keywordOKForFunction.
    def exitKeywordOKForFunction(self, ctx:XQueryParser.KeywordOKForFunctionContext):
        pass


    # Enter a parse tree produced by XQueryParser#uriLiteral.
    def enterUriLiteral(self, ctx:XQueryParser.UriLiteralContext):
        pass

    # Exit a parse tree produced by XQueryParser#uriLiteral.
    def exitUriLiteral(self, ctx:XQueryParser.UriLiteralContext):
        pass


    # Enter a parse tree produced by XQueryParser#stringLiteralQuot.
    def enterStringLiteralQuot(self, ctx:XQueryParser.StringLiteralQuotContext):
        pass

    # Exit a parse tree produced by XQueryParser#stringLiteralQuot.
    def exitStringLiteralQuot(self, ctx:XQueryParser.StringLiteralQuotContext):
        pass


    # Enter a parse tree produced by XQueryParser#stringLiteralApos.
    def enterStringLiteralApos(self, ctx:XQueryParser.StringLiteralAposContext):
        pass

    # Exit a parse tree produced by XQueryParser#stringLiteralApos.
    def exitStringLiteralApos(self, ctx:XQueryParser.StringLiteralAposContext):
        pass


    # Enter a parse tree produced by XQueryParser#stringLiteral.
    def enterStringLiteral(self, ctx:XQueryParser.StringLiteralContext):
        pass

    # Exit a parse tree produced by XQueryParser#stringLiteral.
    def exitStringLiteral(self, ctx:XQueryParser.StringLiteralContext):
        pass


    # Enter a parse tree produced by XQueryParser#stringContentQuot.
    def enterStringContentQuot(self, ctx:XQueryParser.StringContentQuotContext):
        pass

    # Exit a parse tree produced by XQueryParser#stringContentQuot.
    def exitStringContentQuot(self, ctx:XQueryParser.StringContentQuotContext):
        pass


    # Enter a parse tree produced by XQueryParser#stringContentApos.
    def enterStringContentApos(self, ctx:XQueryParser.StringContentAposContext):
        pass

    # Exit a parse tree produced by XQueryParser#stringContentApos.
    def exitStringContentApos(self, ctx:XQueryParser.StringContentAposContext):
        pass


    # Enter a parse tree produced by XQueryParser#noQuotesNoBracesNoAmpNoLAng.
    def enterNoQuotesNoBracesNoAmpNoLAng(self, ctx:XQueryParser.NoQuotesNoBracesNoAmpNoLAngContext):
        pass

    # Exit a parse tree produced by XQueryParser#noQuotesNoBracesNoAmpNoLAng.
    def exitNoQuotesNoBracesNoAmpNoLAng(self, ctx:XQueryParser.NoQuotesNoBracesNoAmpNoLAngContext):
        pass



del XQueryParser


import html
import io
from decimal import Decimal
from typing import Any, Dict, Tuple

from antlr4 import InputStream, CommonTokenStream
from antlr4.error.ErrorListener import ErrorListener
from antlr4.tree.Tree import TerminalNodeImpl

from drb import DrbNode
from drb.factory import DrbFactoryResolver

from .XQueryLexer import XQueryLexer
from .XQueryParser import XQueryParser
from .XQueryParserVisitor import XQueryParserVisitor
from .drb_xquery_context import AxeNode, \
    DynamicContext, NavigationStep, StaticContext, NamespaceMap
from .drb_xquery_func import DrbQueryFuncCall
from .drb_xquery_item import DrbXqueryItem
from .drb_xquery_utils import DrbQueryFuncUtil
from .drb_xquery_variable import XqueryVariable, XqueryBuildIn
from .execptions import DynamicException, ErrorXQUERY, \
    StaticException


class DrbQueryVisitor(XQueryParserVisitor):

    def __init__(self, dynamic_context: DynamicContext,
                 static_context=None,
                 list_context=None,
                 step_nav=NavigationStep.CHILD,
                 is_module=False,
                 external_var_map: dict = None,
                 tokens=None):

        self.is_module = is_module

        self.prefix_module = ''
        self.tokens = tokens

        self.dynamic_context = dynamic_context

        if list_context is None:
            self.list_context = [self.dynamic_context]
        else:
            self.list_context = list_context

        self.current_axe = AxeNode(step_nav)
        self.flwor_scope_list = None

        if static_context is None:
            self.static_context = StaticContext()
            DrbQueryFuncCall.init_namespace_context(self.static_context)
        else:
            self.static_context = static_context

        self.external_var_map = external_var_map
        if self.external_var_map is None:
            self.external_var_map = {}

    def get_namespace_full_name(self, prefix: str):
        ns_full = self.static_context.get_namespace_full_name(prefix)
        if self.dynamic_context and len(ns_full) == 0:
            ns_full = self.dynamic_context.get_namespace_full_name(prefix)
        return ns_full

    def visitVersionDecl(self, ctx: XQueryParser.VersionDeclContext):
        # unused we don't control the version
        return ''

    def visitModuleImport(self, ctx: XQueryParser.ModuleImportContext):
        index_uri = 0
        prefix_module = ''
        ns_module = ''
        uri_of_lib = None
        if ctx.ncName() is not None:
            prefix_module = self.visitNcName(ctx.ncName())
            module_context = self.visitUriLiteral(ctx.uriLiteral(index_uri))
            index_uri = index_uri + 1
            self.static_context.add_namespace(prefix_module, module_context)
            uri_of_lib = module_context
            ns_module = module_context

        if ctx.KW_AT() is not None:
            uri_of_lib = self.visitUriLiteral(ctx.uriLiteral(index_uri))

        if uri_of_lib is not None:
            if uri_of_lib in self.static_context.imported_module.keys():
                return

            self.static_context.imported_module[uri_of_lib] = prefix_module

            node_module = DrbFactoryResolver().create(uri_of_lib)
            if node_module.has_impl(io.RawIOBase):
                impl = node_module.get_impl(io.BufferedIOBase)

                xquery_buffer = impl.readlines()
                xquery_string = ''
                for line in xquery_buffer:
                    xquery_string = xquery_string + line.decode('utf-8')
                lexer = XQueryLexer(InputStream(xquery_string))
                stream = CommonTokenStream(lexer)
                parser = XQueryParser(stream)

                tree = parser.libraryModule()

                visitor = DrbQueryVisitor(None, is_module=True)
                visitor.static_context.imported_module = \
                    self.static_context.imported_module.copy()
                # visitor.static_context.clone(self.static_context)
                output = visitor.visit(tree)

                for ns, name_var in \
                        visitor.static_context.in_scope_variables.keys():
                    var = \
                        visitor.static_context.in_scope_variables[ns, name_var]
                    if ns is not None \
                            and ns != ns_module:
                        continue
                        # TODO raise error
                    var.namespace = prefix_module
                    self.static_context.in_scope_variables[module_context,
                                                           var.name] = var

                for ns, func_name in visitor.static_context.\
                        namespace_builtin_func.keys():

                    if ns is not None and ns != ns_module:
                        continue

                    ns = ns_module
                    func = visitor.static_context.namespace_builtin_func[
                        ns, func_name]

                    self.static_context.namespace_builtin_func[
                        ns, func_name] = func
        else:
            raise DynamicException(ErrorXQUERY.FOAR0001,
                                   "Error import is not correct")

    def visitMainModule(self, ctx: XQueryParser.MainModuleContext):
        if ctx.prolog() is not None:
            visitor = DrbQueryVisitor(self.dynamic_context,
                                      static_context=self.static_context,
                                      external_var_map=self.external_var_map,
                                      tokens=self.tokens)
            visitor.visitProlog(ctx.prolog())
        if ctx.queryBody() is not None:
            return self.visitQueryBody(ctx.queryBody())
        return ''

    def visitModuleDecl(self, ctx: XQueryParser.ModuleDeclContext):
        if ctx.ncName() is not None:
            prefix = self.visitNcName(ctx.ncName())
            full_name = self.visitStringLiteral(ctx.stringLiteral())
            self.static_context.add_namespace(prefix, full_name)
            self.prefix_module = prefix
        return ''

    def visitFunctionDecl(self, ctx: XQueryParser.FunctionDeclContext):
        if ctx.eqName() is not None:
            eqname = self.visitEqName(ctx.eqName())
            if self.is_module:
                self.static_context.add_func_build_in(eqname, ctx,
                                                      self.static_context)
            else:
                self.static_context.add_func_build_in(eqname, ctx)

    def visitNamespaceDecl(self, ctx: XQueryParser.NamespaceDeclContext):
        if ctx.ncName() is not None:
            prefix = self.visitNcName(ctx.ncName())
            full_name = self.visitUriLiteral(ctx.uriLiteral())

            self.static_context.add_namespace(prefix, full_name)
        return ''

    def visitDefaultNamespaceDecl(
            self, ctx: XQueryParser.DefaultNamespaceDeclContext):
        if ctx.stringLiteral() is not None:
            namespace = self.visitStringLiteral(ctx.stringLiteral())
            if ctx.KW_ELEMENT():
                self.static_context.namespace_default_elt = namespace
            if ctx.KW_FUNCTION():
                self.static_context.namespace_default_func = namespace
        return ''

    def visitVarDecl(self, ctx: XQueryParser.VarDeclContext):
        varname_full = self.visitVarName(ctx.varName())
        var_namespace, var_name = DrbQueryFuncUtil.split_namespace_name(
            varname_full)

        var_value = None
        external_var = False
        type_var = None
        if ctx.typeDeclaration() is not None:
            type_var = self.visitTypeDeclaration(ctx.typeDeclaration())

        if ctx.KW_EXTERNAL() is not None:
            external_var = True
            if var_name in self.external_var_map.keys():
                var_value = self.external_var_map[var_name]
            elif ctx.varDefaultValue() is not None:
                var_value = self.visitVarDefaultValue(ctx.varDefaultValue())
            else:
                raise DynamicException(
                    ErrorXQUERY.XPDY0002,
                    "No value for external variable " + var_name + " given.")

        elif ctx.varValue() is not None:
            var_value = self.visitVarValue(ctx.varValue())

        self.static_context.add_var(varname_full,
                                    XqueryVariable(var_name,
                                                   var_namespace,
                                                   external=external_var,
                                                   value=var_value,
                                                   type_var=type_var))

    def visitVarRef(self, ctx: XQueryParser.VarRefContext):
        var_name = self.visitEqName(ctx.eqName())
        var = self.static_context.get_var(var_name)

        return var.get_value()

    def visitStringConcatExpr(self, ctx: XQueryParser.StringConcatExprContext):
        res = self.visitChildren(ctx)
        return res

    def visitSequenceType(self, ctx: XQueryParser.SequenceTypeContext):
        res = self.visitItemType(ctx.itemType())
        if res is None:
            return res
        if ctx.STAR():
            # to simplify if max > 1 like 100 there are no limit
            return res, 0, 100
        if ctx.PLUS():
            # to simplify if max > 1 like 100 there are no limit
            return res, 1, 100
        if ctx.QUESTION():
            return res, 0, 1

        return res

    def visitArgumentList(self, ctx: XQueryParser.ArgumentListContext):
        result = []
        list_context = self.list_context
        dynamic_context = self.dynamic_context
        for arg in ctx.argument():
            result.append(self.visitArgument(arg))
            self.list_context = list_context
            self.dynamic_context = dynamic_context

        return result

    @staticmethod
    def get_value_from_definition_type(value_arg,
                                       type_var_full,
                                       func_msg):
        arg_min = 1
        arg_max = 1
        if isinstance(type_var_full, tuple):
            arg_min = type_var_full[1]
            arg_max = type_var_full[2]
            type_var_full = type_var_full[0]

        type_var_namespace, type_var = \
            DrbQueryFuncUtil.split_namespace_name(type_var_full)
        value_arg_typed = XqueryVariable.get_value_typed(
            type_var,
            type_var_namespace,
            value_arg,
            arg_min=arg_min,
            arg_max=arg_max,
            func_msg=func_msg)
        return value_arg_typed

    def call_built_in_func(self,
                           func: XqueryBuildIn,
                           argument_list: list):
        built_in_func = func.exp_func
        list_param = built_in_func.functionParams()

        old_scop = None
        previous_context = None
        if func.specific_context is not None:
            previous_context = self.static_context
            self.static_context = func.specific_context
        old_scop = self.static_context.in_scope_variables
        self.static_context.in_scope_variables = old_scop.copy()

        if list_param is not None:
            list_def_args = list_param.functionParam()
            if len(list_def_args) != len(argument_list):
                raise DynamicException(ErrorXQUERY.XPST0017, "Function call " +
                                       func.name +
                                       "() does not match "
                                       "arity of "
                                       "the built-in function")
            for index in range(len(list_def_args)):
                def_arg = list_def_args[index]
                arg_name = self.visitQName(def_arg.qName())
                value_arg = argument_list[index]

                if def_arg.typeDeclaration() is not None:
                    type_var_full = self.visitTypeDeclaration(
                        def_arg.typeDeclaration())
                    if type_var_full is not None:
                        value_arg = self.get_value_from_definition_type(
                            value_arg,
                            type_var_full,
                            func.name + ' arg index ' + str(index + 1))

                self.static_context.add_var(arg_name, XqueryVariable(arg_name,
                                            value=value_arg))
        elif len(argument_list) != 0:
            raise DynamicException(ErrorXQUERY.XPST0017, "Function call " +
                                   func.name +
                                   "() is call with arg "
                                   "but build in don't take any "
                                   "any built-in function")

        result = self.visitFunctionBody(built_in_func.functionBody())

        if built_in_func.functionReturn() is not None and \
                built_in_func.functionReturn().KW_AS() is not None:
            type_var_full = self.visitSequenceType(built_in_func.
                                                   functionReturn().
                                                   sequenceType())

            if type_var_full is not None:
                result = self.get_value_from_definition_type(result,
                                                             type_var_full,
                                                             func.name +
                                                             ' return ')

        self.static_context.in_scope_variables = old_scop

        if previous_context is not None:
            self.static_context = previous_context

        return result

    def visitFunctionCall(self, ctx: XQueryParser.FunctionCallContext):
        name_full = self.visitEqName(ctx.eqName())

        (namespace, name_func) = DrbQueryFuncUtil.split_namespace_name(
            name_full)

        if namespace is None or len(namespace) == 0:
            namespace = self.static_context.namespace_default_func

        func_to_call = self.static_context.get_func_namespace(namespace,
                                                              name_func)
        argument_list = self.visitArgumentList(ctx.argumentList())

        if func_to_call is not None:
            return func_to_call(
                name_func, self.dynamic_context,
                self.static_context, *argument_list)

        func_to_call = self.static_context.get_func_build_in(name_full)
        if func_to_call is not None:
            return self.call_built_in_func(func_to_call, argument_list)

        raise DynamicException(ErrorXQUERY.XPST0017, "Function call " +
                               name_full +
                               "() does not match the name and "
                               "arity of "
                               "any built-in function")

    def visitAttributeNameOrWildcard(self,
                                     ctx: XQueryParser
                                     .AttributeNameOrWildcardContext):
        return self.visitChildren(ctx)

    def visitElementNameOrWildcard(self,
                                   ctx: XQueryParser
                                   .ElementNameOrWildcardContext):
        return self.visitChildren(ctx)

    @staticmethod
    def append_or_extend(object_to_append, list_object: list):
        if isinstance(object_to_append, list):
            list_object.extend(object_to_append)
        else:
            list_object.append(object_to_append)

    @staticmethod
    def append_object_or_index(list_source: list, list_or_slice_or_index,
                               list_result: list):
        if isinstance(list_or_slice_or_index, list):
            list_result.extend(list_or_slice_or_index)
        elif isinstance(list_or_slice_or_index, int):
            list_result.append(list_source[list_or_slice_or_index])
        else:
            list_result.append(list_or_slice_or_index)

    def visitOrExpr(self, ctx: XQueryParser.OrExprContext):
        if len(ctx.KW_OR()) == 0:
            return self.visitChildren(ctx)

        for toComp in ctx.andExpr():
            visitor_for_comp = DrbQueryVisitor(
                self.dynamic_context,
                static_context=self.static_context,
                external_var_map=self.external_var_map,
                tokens=self.tokens)
            result = visitor_for_comp.visitAndExpr(toComp)
            if DrbQueryFuncUtil.get_effective_boolean_value(result):
                return True
        return False

    def visitAndExpr(self, ctx: XQueryParser.AndExprContext):
        if len(ctx.KW_AND()) == 0:
            return self.visitChildren(ctx)

        for toComp in ctx.comparisonExpr():
            visitor_for_comp = DrbQueryVisitor(
                self.dynamic_context,
                static_context=self.static_context,
                external_var_map=self.external_var_map,
                tokens=self.tokens)
            result = visitor_for_comp.visitComparisonExpr(toComp)
            if not DrbQueryFuncUtil.get_effective_boolean_value(result):
                return False

        return True

    def visitComparisonExpr(self, ctx: XQueryParser.ComparisonExprContext):
        op = ctx.valueComp()
        if op is None:
            op = ctx.generalComp()
        if op is None:
            op = ctx.nodeComp()
        if op is None:
            return self.visitChildren(ctx)
        op = op.getText()

        visitor_for_left = DrbQueryVisitor(self.dynamic_context,
                                           static_context=self.static_context,
                                           external_var_map=self.
                                           external_var_map,
                                           tokens=self.tokens)
        visitor_for_right = DrbQueryVisitor(self.dynamic_context,
                                            static_context=self.static_context,
                                            external_var_map=self.
                                            external_var_map,
                                            tokens=self.tokens)

        left = visitor_for_left.visitStringConcatExpr(
            ctx.stringConcatExpr(0))
        right = visitor_for_right.visitStringConcatExpr(
            ctx.stringConcatExpr(1))

        list_left = left
        if not isinstance(left, list):
            list_left = [left]

        list_right = right
        if not isinstance(right, list):
            list_right = [right]

        test_ok = False

        for left_item in list_left:
            for right_item in list_right:
                if op == 'is':
                    if left_item is right_item:
                        test_ok = True
                        break
                    else:
                        continue

                left_value, right_value = \
                    DrbQueryFuncUtil.convert_before_compare(left_item,
                                                            right_item)

                if op == 'eq' or op == '=':
                    if DrbQueryFuncUtil.is_equal(left_value, right_value):
                        test_ok = True
                        break
                elif op == 'ne' or op == '!=':
                    if left_value != right_value:
                        test_ok = True
                        break
                elif op == 'gt' or op == '>':
                    if left_value > right_value:
                        test_ok = True
                        break
                elif op == 'ge' or op == '>=':
                    if left_value >= right_value:
                        test_ok = True
                        break
                elif op == 'lt' or op == '<':
                    if left_value < right_value:
                        test_ok = True
                        break
                elif op == 'le' or op == '<=':
                    if left_value <= right_value:
                        test_ok = True
                        break

            if test_ok:
                break

        return test_ok

    def visitUnaryExpression(self, ctx: XQueryParser.UnaryExpressionContext):
        res = self.visitChildren(ctx)
        if ctx.MINUS() is not None and len(ctx.MINUS()) > 0:
            if isinstance(res, (int, Decimal, float)):
                res = res * -1
        return res

    def visitNumericLiteral(self, ctx: XQueryParser.NumericLiteralContext):
        res = ctx.IntegerLiteral()
        if res is not None:
            return int(ctx.getText())
        res = ctx.DoubleLiteral()
        if res is not None:
            return float(ctx.getText())
        res = ctx.DecimalLiteral()
        if res is not None:
            return Decimal(ctx.getText())
        return ctx.getText()

    def visitStringLiteral(self, ctx: XQueryParser.StringLiteralContext):
        if ctx.stringLiteralQuot() is not None:
            # for item in ctx.stringLiteralQuot().stringContentQuot():
            result = ''
            for stringContext in ctx.stringLiteralQuot().children:
                str_Quot = stringContext.getText()
                if str_Quot == '"':
                    continue
                if str_Quot == '""':
                    str_Quot = '"'
                result = result + str(str_Quot)
            return result
        if ctx.stringLiteralApos() is not None:
            result = ''
            for stringContext in ctx.stringLiteralApos().stringContentApos():
                str_Quot = stringContext.getText()
                if str_Quot == "'":
                    continue
                result = result + str_Quot
            return result
        return self.visitChildren(ctx)

    def visitIntersectExceptExpr(self,
                                 ctx: XQueryParser.IntersectExceptExprContext):
        if ctx.KW_INTERSECT() and len(ctx.KW_INTERSECT()) >= 1:
            result = None
            for exp in ctx.instanceOfExpr():
                result_partial = self.visitInstanceOfExpr(exp)
                result = DrbQueryFuncUtil.intersect(result_partial,
                                                    result)
            return result
        elif ctx.KW_EXCEPT() and len(ctx.KW_EXCEPT()) >= 1:
            result = None
            for exp in ctx.instanceOfExpr():
                result_partial = self.visitInstanceOfExpr(exp)

                result = DrbQueryFuncUtil.except_op(result_partial,
                                                    result)
            return result
        else:
            return self.visitChildren(ctx)

    def visitUnionExpr(self, ctx: XQueryParser.UnionExprContext):
        if len(ctx.intersectExceptExpr()) <= 1:
            return self.visitChildren(ctx)

        result = None
        for exp in ctx.intersectExceptExpr():
            res_partial = self.visitIntersectExceptExpr(exp)
            result = DrbQueryFuncUtil.union(res_partial, result)
        return result

    def visitMultiplicativeExpr(self,
                                ctx: XQueryParser.MultiplicativeExprContext):
        if len(ctx.unionExpr()) <= 1:
            return self.visitChildren(ctx)

        operation = None

        res_final = None
        n = ctx.getChildCount()
        for i in range(n):

            c = ctx.getChild(i)
            if isinstance(c, TerminalNodeImpl):
                operation = c.getText()
            elif isinstance(c, XQueryParser.UnionExprContext):
                left_part = self.visitUnionExpr(c)
                operand_left = DrbQueryFuncUtil.get_numeric_value(
                    left_part, True)

                if operand_left is None:
                    if res_final is None:
                        raise StaticException(ErrorXQUERY.XPST0003,
                                              "first opreand of "
                                              "Multiplicative Exp "
                                              "must be numeric")
                    else:
                        operand_left = 0

                if res_final is None:
                    res_final = operand_left
                elif operation == '*':
                    res_final = res_final * operand_left
                elif operation == 'mod':
                    res_final = res_final % operand_left
                elif operation == 'div':
                    # Manage div by zero
                    if operand_left == 0:
                        # if float can return inf or -inf
                        if isinstance(res_final, float) \
                                or isinstance(operand_left, float):
                            if res_final == 0:
                                res_final = float('nan')
                            elif res_final > 0:
                                res_final = float('inf')
                            else:
                                res_final = float('-inf')
                        # otherwise, it is an error
                        else:
                            raise DynamicException(ErrorXQUERY.FOAR0001,
                                                   " Try to divide "
                                                   + str(res_final)
                                                   + " by zero")
                    else:
                        res_final = res_final / operand_left
                # int div
                elif operation == 'idiv':
                    if operand_left == 0:
                        raise DynamicException(ErrorXQUERY.FOAR0001,
                                               " Try to divide "
                                               + str(res_final)
                                               + " by zero")
                    else:
                        res_final = res_final // operand_left

        return res_final

    def visitAdditiveExpr(self, ctx: XQueryParser.AdditiveExprContext):
        if len(ctx.multiplicativeExpr()) <= 1:
            return self.visitChildren(ctx)

        operation = None

        res_final = None
        n = ctx.getChildCount()
        for i in range(n):

            c = ctx.getChild(i)
            if isinstance(c, TerminalNodeImpl):
                operation = c.getText()
            elif isinstance(c, XQueryParser.MultiplicativeExprContext):
                left_part = self.visitMultiplicativeExpr(c)
                operand_left = DrbQueryFuncUtil.get_numeric_value(left_part,
                                                                  True)

                if res_final is None:
                    res_final = operand_left
                elif operation == '+':
                    res_final = res_final + operand_left
                elif operation == '-':
                    res_final = res_final - operand_left

        return res_final

    def visitAbbrevReverseStep(self,
                               ctx: XQueryParser.AbbrevReverseStepContext):
        if ctx.DDOT() is not None:
            self.current_axe.is_parent = True

        return ''

    def visitAbbrevForwardStep(self,
                               ctx: XQueryParser.AbbrevForwardStepContext):
        at = ctx.AT()
        res = ''
        if at is not None:
            res = str(at)
            self.current_axe.is_attribute = True
            self.current_axe.is_node = False

        name = self.visitChildren(ctx)
        if name is not None:
            res = res + name

        return res

    def visitEqName(self, ctx: XQueryParser.EqNameContext):
        return self.visitChildren(ctx)

    def visitNameTest(self, ctx: XQueryParser.NameTestContext):
        if ctx.wildcard() is not None:
            self.current_axe.is_wildcard = True
            return '*'
        return self.visitChildren(ctx)

    def visitQName(self, ctx: XQueryParser.QNameContext):
        if ctx.ncName() is not None:
            return self.visitChildren(ctx)

        if ctx.FullQName() is not None:
            test = ctx.getText().split(':')

            prefix_namespace = test[0]
            self.current_axe.namespace = self.get_namespace_full_name(
                prefix_namespace)
            if len(self.current_axe.namespace) == 0:
                self.current_axe.namespace = prefix_namespace
            self.current_axe.name = test[1]

            return ctx.getText()

        return self.visitChildren(ctx)

    def visitNcName(self, ctx: XQueryParser.NCName):
        self.current_axe.name = ctx.getText()

        return ctx.getText()

    def visitRangeExpr(self, ctx: XQueryParser.RangeExprContext):
        if ctx.KW_TO() is None:
            return self.visitChildren(ctx)

        left = self.visitAdditiveExpr(ctx.additiveExpr(0))
        right = self.visitAdditiveExpr(ctx.additiveExpr(1))

        left = DrbQueryFuncUtil.get_numeric_value(left)
        right = DrbQueryFuncUtil.get_numeric_value(right)

        res = list(range(left, right + 1))
        return res

    def visit_the_list_of_predicate(self,  predicate_list, context_list):
        self.static_context.is_in_predicate = True
        result = context_list
        for predicate_exp in predicate_list:
            res_predicate = self.visitPredicateContext(predicate_exp,
                                                       context_list)
            result = []

            DrbQueryVisitor.append_object_or_index(
                context_list, res_predicate, result)

            context_list = result
        self.static_context.is_in_predicate = False
        return result

    def visitCommonContent(self, ctx: XQueryParser.CommonContentContext):
        if ctx.expr() is not None:
            res = self.visitExpr(ctx.expr())
        else:
            res = html.unescape(ctx.getText())
            res = res.replace("{{", "{")
            res = res.replace("}}", "}")

        return res

    def visitDirAttributeValueApos(self, ctx: XQueryParser.
                                   DirAttributeValueAposContext):
        result = ''
        index = 0
        if ctx.children is not None:
            for item in ctx.children:
                index = index + 1

                if isinstance(item,
                              XQueryParser.DirAttributeContentQuotContext) \
                        and item.expr() is not None:
                    result = result + \
                             DrbQueryFuncUtil.get_string(
                                 self.visitExpr(item.expr()))
                else:
                    if not ((index == 1 or index == len(ctx.children)) and
                            (item.getText() == '"' or item.getText() == "'")):
                        result = result + item.getText()
        if result.find('<') >= 0:
            raise StaticException(ErrorXQUERY.XPST0003,
                                  "char '<' can not be in attributes values")
        return result

    def visitDirAttributeValueQuot(self, ctx: XQueryParser.
                                   DirAttributeValueQuotContext):
        result = ''
        index = 0

        if ctx.children is not None:
            for item in ctx.children:
                index = index + 1

                if isinstance(item,
                              XQueryParser.DirAttributeContentAposContext) \
                        and item.expr() is not None:
                    result = result + \
                             DrbQueryFuncUtil.get_string(
                                 self.visitExpr(item.expr()))
                else:
                    if not ((index == 1 or index == len(ctx.children)) and
                            (item.getText() == '"' or item.getText() == "'")):
                        result = result + item.getText()
        if result.find('<') >= 0:
            raise StaticException(ErrorXQUERY.XPST0003,
                                  "char '<' can not be in attributes values")
        return result

    def getAttributeValueList(self,
                              ctx: XQueryParser.DirAttributeListContext,
                              node: DrbXqueryItem):
        attributes: Dict[Tuple[str, str], Any] = {}
        if ctx.children is not None:
            attribute_full_name = ''
            for item in ctx.children:
                if isinstance(item, XQueryParser.QNameContext):
                    attribute_full_name = self.visitQName(item)
                elif isinstance(item, XQueryParser.DirAttributeValueContext):
                    namespace, attribute_name = DrbQueryFuncUtil.\
                        split_namespace_name(attribute_full_name)
                    value_attribute = self.visitDirAttributeValue(item)

                    if namespace == 'xmlns':
                        ns_full = value_attribute.strip('"')
                        self.dynamic_context.add_namespace(
                            self.static_context,
                            attribute_name,
                            ns_full)
                    else:
                        if attribute_name == 'xmlns':
                            if node.namespace_uri is None \
                                    or len(node.namespace_uri) == 0:
                                node._namespace_uri = value_attribute.\
                                    strip('"')
                                NamespaceMap.check_is_predefined_namespace(
                                    None, node._namespace_uri)

                                self.dynamic_context.namespace_default_elt = \
                                    node._namespace_uri
                        else:
                            if namespace is not None:
                                namespace = self.\
                                    get_namespace_full_name(namespace)
                            attributes[attribute_name, namespace] = \
                                value_attribute
        return attributes

    def visitNoQuotesNoBracesNoAmpNoLAng(self,
                                         ctx:
                                         XQueryParser.
                                         NoQuotesNoBracesNoAmpNoLAngContext):
        result = ''
        index_previous = ctx.start.tokenIndex - 1

        if self.tokens is not None:
            previous = self.tokens.get(index_previous)
            while previous.channel == 2:
                result = previous.text + result
                index_previous = index_previous - 1
                previous = self.tokens.get(index_previous)

            for index in range(ctx.start.tokenIndex, ctx.stop.tokenIndex+1):
                test_tokens = self.tokens.get(index)
                result = result + test_tokens.text

            index_after = ctx.stop.tokenIndex+1
            after = self.tokens.get(index_after)
            while after.channel == 2:
                result = result + after.text
                index_after = index_after + 1
                after = self.tokens.get(index_after)
        result = html.unescape(result)

        return result

    def visitDirElemContent(self, ctx: XQueryParser.DirElemContentContext):
        if ctx.noQuotesNoBracesNoAmpNoLAng() is not None:
            return self.visitNoQuotesNoBracesNoAmpNoLAng(
                ctx.noQuotesNoBracesNoAmpNoLAng())
        elif ctx.directConstructor() is not None:
            return self.visitDirectConstructor(ctx.directConstructor())
        # elif ctx.CDATA() is not None:
        #     node = XmlNode(XML(ctx.getText()))
        #     return DynamicContext(node)
        return self.visitChildren(ctx)

    def get_elem_cons_name_and_attributes(self, ctx, q_name: str):
        (namespace, name) = DrbQueryFuncUtil. \
            split_namespace_name(q_name)

        namespace_full = None
        node = DrbXqueryItem(None, name, namespace, namespace_full)
        self.dynamic_context = DynamicContext(
            node, dynamic_context_prev=self.dynamic_context)

        if ctx.dirAttributeList() is not None:
            attributes_node = self.getAttributeValueList(
                ctx.dirAttributeList(), node)
            node.set_attribute(attributes_node)

        if namespace is not None:
            node._namespace_uri = self.get_namespace_full_name(namespace)
        elif node.namespace_uri is None and \
                len(self.dynamic_context.namespace_default_elt) > 0:
            node._namespace_uri = self.dynamic_context.namespace_default_elt

        return node

    def visitDirElemConstructorOpenClose(self, ctx: XQueryParser.
                                         DirElemConstructorOpenCloseContext):
        if len(ctx.qName()) == 0:
            raise StaticException(ErrorXQUERY.XPST0003,
                                  "Name expected for direct contructor")

        dynamic_context_old = self.dynamic_context

        q_name = ctx.qName(0).getText()
        if len(ctx.qName()) > 1:
            q_name_end = ctx.qName(1).getText()
            if q_name_end != q_name:
                raise StaticException(ErrorXQUERY.XPST0003,
                                      "The name " + q_name + " of element " +
                                      " tag constructor  differ " +
                                      q_name_end)

        node = self.get_elem_cons_name_and_attributes(ctx, q_name)

        value = None
        for item in ctx.dirElemContent():
            result_value = self.visitDirElemContent(item)
            if not isinstance(result_value, list):
                result_value = [result_value]

            value_content = None
            for item_value in result_value:
                node.order_elt.append(item_value)
                if isinstance(item_value, DrbNode):
                    node.append_child(item_value)
                if isinstance(item_value, DynamicContext):
                    if item_value.is_attribute():
                        node.add_attribute(item_value.name,
                                           item_value.value,
                                           item_value.namespace_uri)
                    else:
                        node.append_child(item_value.node)
                elif value_content is None:
                    value_content = item_value
                else:
                    value_content = \
                        DrbQueryFuncUtil.get_string(value_content) + ' ' + \
                        DrbQueryFuncUtil.get_string(item_value)

            if value_content is not None:
                if value is None:
                    value = value_content
                else:
                    value = DrbQueryFuncUtil.get_string(value) + \
                            DrbQueryFuncUtil.get_string(value_content)
        node.set_value(value)

        context_to_return = self.dynamic_context

        self.dynamic_context = dynamic_context_old

        return context_to_return

    def visitDirElemConstructorSingleTag(self, ctx: XQueryParser.
                                         DirElemConstructorSingleTagContext):
        if ctx.qName() is None:
            return ''
        dynamic_context_old = self.dynamic_context

        node = self.get_elem_cons_name_and_attributes(ctx,
                                                      ctx.qName().getText())

        context_to_return = self.dynamic_context
        self.dynamic_context = dynamic_context_old

        return context_to_return

    def visitDirectConstructor(self, ctx: XQueryParser.
                               DirectConstructorContext):
        if self.dynamic_context.namespace_default_elt is None or \
                len(self.dynamic_context.namespace_default_elt) == 0:
            self.dynamic_context.namespace_default_elt = \
                self.static_context.namespace_default_elt

        if ctx.dirElemConstructorSingleTag() is not None:
            result = self.visitDirElemConstructorSingleTag(
                ctx.dirElemConstructorSingleTag())
        elif ctx.dirElemConstructorOpenClose() is not None:
            result = self.visitDirElemConstructorOpenClose(
                ctx.dirElemConstructorOpenClose())
        else:
            if ctx.getText().startswith('<!--') and\
                    ctx.getText().endswith('-->'):
                return None
            raise StaticException(ErrorXQUERY.XPST0003,
                                  "unable to find constructor" + ctx.getText())

        return result

    def visitContextItemExpr(self, ctx: XQueryParser.ContextItemExprContext):
        if ctx.DOT() is not None:
            return self.dynamic_context
        return self.visitChildren(ctx)

    def visitPredicate(self, ctx: XQueryParser.PredicateContext):
        return self.visitPredicateContext(ctx, self.list_context)

    def visitPredicateContext(self,
                              ctx: XQueryParser.PredicateContext,
                              list_context):
        # visitor = DrbQueryVisitor(self.dynamic_context, self.position)
        result = []
        index = 0
        for node in list_context:
            if isinstance(node, DynamicContext):
                nodeToVisit = node
            else:
                nodeToVisit = DynamicContext(node)

            nodeToVisit.size_context = len(list_context)
            nodeToVisit.position = index + 1

            visitor_exp = DrbQueryVisitor(nodeToVisit,
                                          static_context=self.static_context,
                                          external_var_map=self.
                                          external_var_map,
                                          tokens=self.tokens)
            res_partial = visitor_exp.visitExpr(ctx.expr())
            index = index + 1

            if isinstance(res_partial, list):
                if len(res_partial) > 0:
                    if isinstance(res_partial[0], DrbNode):
                        result.append(node)
                    elif isinstance(res_partial[0], DynamicContext) and \
                            res_partial[0].is_node():
                        result.append(node)
                    elif isinstance(res_partial[0], DynamicContext) and \
                            res_partial[0].is_attribute():
                        result.append(node)
                    else:
                        list_numeric = [s for s in res_partial
                                        if isinstance(s,
                                                      (int, float, Decimal))]
                        if len(list_numeric) == len(res_partial):
                            if index in res_partial:
                                result.append(node)
                        else:
                            raise DynamicException(
                               ErrorXQUERY.FORG0006,
                               "A sequence staring by other than "
                               "a node can not be used as predicate result")

            elif isinstance(res_partial, bool):
                if res_partial:
                    result.append(node)
            elif isinstance(res_partial, (int, float, Decimal)):
                if index == res_partial:
                    result.append(node)
            elif isinstance(res_partial, str):
                if len(res_partial) > 0:
                    result.append(node)
            elif res_partial is not None:
                result.append(node)

        return result

    def visitExprForName(self, ctx: XQueryParser.ExprContext):
        result = self.visitExpr(ctx)
        return DrbQueryFuncUtil.get_name_in_result(result)

    def visitExpr(self, ctx: XQueryParser.ExprContext):
        if ctx.COMMA() is None or len(ctx.COMMA()) == 0:
            return self.visitChildren(ctx)

        result = []

        list_context = self.list_context
        for single_exp in ctx.exprSingle():
            res_partial = self.visitExprSingle(single_exp)
            self.append_or_extend(res_partial, result)
            self.list_context = list_context
        return result

    # Visit a parse tree produced by XQueryParser#parenthesizedExpr.
    def visitParenthesizedExpr(self,
                               ctx: XQueryParser.ParenthesizedExprContext):
        if ctx.expr() is not None:
            return self.visitExpr(ctx.expr())
        return []

    def visitPostfixExpr(self, ctx: XQueryParser.PostfixExprContext):
        if len(ctx.predicate()) == 0:
            return self.visitChildren(ctx)

        result = self.visitPrimaryExpr(ctx.primaryExpr())

        predicate_list = ctx.predicate()
        if predicate_list is not None and len(predicate_list) > 0:
            if isinstance(result, list):
                list_context = result
            else:
                list_context = [result]

            result = self.visit_the_list_of_predicate(predicate_list,
                                                      list_context)

        return result

    def visitForwardAxis(self, ctx: XQueryParser.ForwardAxisContext):
        if ctx.KW_CHILD() is not None:
            self.current_axe.navigation_step = NavigationStep.CHILD
            return ''
        if ctx.KW_ATTRIBUTE() is not None:
            self.current_axe.is_attribute = True
            self.current_axe.is_node = False
            return ''
        if ctx.KW_DESCENDANT_OR_SELF():
            self.current_axe.navigation_step = \
                NavigationStep.DESCENDANT_OR_SELF
            return ''
        if ctx.KW_DESCENDANT():
            self.current_axe.navigation_step = \
                NavigationStep.DESCENDANT
            return ''
        raise NotImplementedError(
            'Unsupported Axis operator :' + ctx.getText())

    def visitReverseAxis(self, ctx: XQueryParser.ReverseAxisContext):
        if ctx.KW_PARENT() is not None:
            self.current_axe.is_parent = True
            return ''

        raise NotImplementedError(
            'Unsupported Axis operator :' + ctx.getText())

    @staticmethod
    def get_attributes(node, current_axe, list_result):
        if current_axe.is_wildcard:
            for key in node.attributes.keys():
                list_result.append(
                    DynamicContext(key, value=node.attributes[key],
                                   parent=node,
                                   attribute=True,
                                   position=len(list_result) + 1))
        else:
            try:
                value_attr = node.get_attribute(
                    current_axe.name,
                    current_axe.namespace)
                if value_attr is not None:
                    list_result.append(DynamicContext(
                        (current_axe.name,
                         current_axe.namespace),
                        value=value_attr, parent=node,
                        attribute=True,
                        position=len(list_result) + 1))
            except Exception as Error:
                # no   attribute with this name.
                pass
        return list_result

    @staticmethod
    def get_children(node, current_axe, list_result):
        next_nodes = []
        if current_axe.is_wildcard:
            next_nodes = node.children
        elif node.has_child(current_axe.name,
                            current_axe.namespace):
            if isinstance(node, DrbXqueryItem):
                next_nodes = node.get_named_child_list(current_axe.name,
                                                       current_axe.namespace)
            else:

                next_nodes = node[current_axe.name,
                                  current_axe.namespace, :]
        # elif node.has_child(current_axe.name,
        #                     None):
        #     if isinstance(node, DrbXqueryItem):
        #         next_nodes = node.get_named_child_list(current_axe.name,
        #                                                None)
        #     else:
        #         next_nodes = node[current_axe.name,
        #                      None, :]
        for node_drb in next_nodes:
            list_result.append(DynamicContext(
                node_drb,
                position=len(list_result) + 1))

        return list_result

    def get_descendant(self, node, current_axe, result, or_self):
        children_of_node = node.children
        if self.current_axe.is_attribute:
            if or_self:
                self.get_attributes(node, current_axe, result)
        else:
            self.get_children(node, self.current_axe, result)

        for child in children_of_node:
            result = self.get_descendant(child, current_axe, result, True)
        return result

    def visitAxisStep(self, ctx: XQueryParser.AxisStepContext):
        list_result = []
        self.current_axe = AxeNode(self.current_axe.navigation_step)

        if ctx.reverseStep() is not None:
            self.visitReverseStep(ctx.reverseStep())
        if ctx.forwardStep() is not None:
            self.visitForwardStep(ctx.forwardStep())

        for context in self.list_context:
            if context is None:
                continue
                # raise DynamicException(ErrorXQUERY.XPTY0020, "Error ")
            if isinstance(context, DynamicContext):
                node = context.node
            else:
                node = context
            next_nodes = []

            if self.current_axe.is_parent:
                if isinstance(node, DrbNode):
                    parent = node.parent
                else:
                    parent = context.parent

                if parent is not None:
                    list_result.append(DynamicContext(
                        parent,
                        position=len(list_result) + 1))
                continue

            if not isinstance(node, DrbNode):
                raise DynamicException(ErrorXQUERY.XPTY0020, "Error ")

            try:
                if self.current_axe.navigation_step == NavigationStep.\
                        DESCENDANT_OR_SELF:
                    if self.current_axe.is_wildcard or \
                            self.current_axe.name == node.name and \
                            (self.current_axe.namespace is None or
                             self.current_axe.namespace == node.namespace_uri):
                        list_result.append(DynamicContext(
                            node,
                            position=len(list_result) + 1))
                    self.get_descendant(node,
                                        self.current_axe,
                                        list_result,
                                        or_self=True)
                elif self.current_axe.navigation_step == NavigationStep. \
                        DESCENDANT:
                    self.get_descendant(node,
                                        self.current_axe,
                                        list_result,
                                        or_self=False)
                elif self.current_axe.is_attribute:
                    self.get_attributes(node, self.current_axe, list_result)
                elif self.current_axe.navigation_step == NavigationStep.CHILD:
                    self.get_children(node, self.current_axe, list_result)
            except Exception as Error:
                # no  node or attribute with this name.
                pass
        list_node_to_test = list_result

        list_predicate = ctx.predicateList()

        if len(list_node_to_test) == 0:
            return []
        if list_predicate is not None:
            list_result = self.visit_the_list_of_predicate(
                list_predicate.predicate(), list_node_to_test
            )

        return list_result

    # Visit a parse tree produced by XQueryParser#relativePathExpr.
    def visitRelativePathExpr(self, ctx: XQueryParser.RelativePathExprContext):
        if (ctx.SLASH is None or len(ctx.SLASH()) == 0) and \
                (ctx.DSLASH() is None or len(ctx.DSLASH()) == 0):
            return self.visitChildren(ctx)

        result = []
        step_nav = NavigationStep.CHILD

        for exp in ctx.children:
            if isinstance(exp, TerminalNodeImpl):
                if exp.getText() == '/':
                    step_nav = NavigationStep.CHILD
                elif exp.getText() == '//':
                    step_nav = NavigationStep.DESCENDANT_OR_SELF
            elif isinstance(exp, XQueryParser.StepExprContext):
                result = []
                index = 0
                for node in self.list_context:

                    visitor_exp = DrbQueryVisitor(
                        node, step_nav=step_nav,
                        static_context=self.static_context,
                        external_var_map=self.external_var_map,
                        tokens=self.tokens)
                    result_partial = visitor_exp.visitStepExpr(exp)
                    index = index + 1
                    if result_partial is not None:
                        if isinstance(result_partial, list):
                            result.extend(result_partial)
                        else:
                            result.append(result_partial)

                self.list_context = result
        return result

        # Visit a parse tree produced by XQueryParser#enclosedContentExpr.

    def visitEnclosedExpression(self,
                                ctx: XQueryParser.EnclosedExpressionContext):
        if ctx.expr() is not None:
            return self.visitExpr(ctx.expr())
        return self.visitChildren(ctx)

    def visitCompAttrConstructor(self,
                                 ctx: XQueryParser.CompAttrConstructorContext):
        if ctx.KW_ATTRIBUTE() is None:
            return self.visitChildren(ctx)

        name = None
        if ctx.eqName() is not None:
            name = self.visitEqName(ctx.eqName())
        elif ctx.expr() is not None:
            name = self.visitExprForName(ctx.expr())
        if name is None:
            raise StaticException(ErrorXQUERY.XPST0003,
                                  "Comp attribute with no name in " +
                                  ctx.getText())

        (namespace, name) = DrbQueryFuncUtil.split_namespace_name(name)

        value_attr = None
        if ctx.enclosedExpression() is not None:
            value_attr = self.visitEnclosedExpression(
                ctx.enclosedExpression())
            value_attr = DrbQueryFuncUtil.get_string(value_attr)

        if namespace is not None:
            namespace = self.get_namespace_full_name(namespace)

        return DynamicContext((name, namespace), value=value_attr,
                              namespace=namespace,
                              attribute=True)

    def visitCompElemConstructor(
            self, ctx: XQueryParser.CompElemConstructorContext):
        if ctx.KW_ELEMENT() is None:
            return self.visitChildren(ctx)
        name = None
        if ctx.eqName() is not None:
            name = self.visitEqName(ctx.eqName())
        elif ctx.expr() is not None:
            name = self.visitExprForName(ctx.expr())

        if name is None:
            raise StaticException(ErrorXQUERY.XPST0003,
                                  "Comp element with no name in " +
                                  ctx.getText())

        (namespace, name) = DrbQueryFuncUtil.split_namespace_name(name)
        namespace_full = None
        if namespace is not None:
            namespace_full = self.get_namespace_full_name(namespace)
        elif len(self.dynamic_context.namespace_default_elt) > 0:
            namespace_full = self.dynamic_context.namespace_default_elt
        node = DrbXqueryItem(None, name, namespace, namespace_full)

        if ctx.enclosedContentExpr() is not None:
            result = self.visitEnclosedContentExpr(ctx.enclosedContentExpr())
            if not isinstance(result, list):
                result = [result]

            value_node = None
            for res_item in result:
                if isinstance(res_item, DynamicContext):
                    if res_item.is_node():
                        res_item.node._parent = self
                        node.append_child(res_item)
                    elif res_item.is_attribute():
                        node.add_attribute(res_item.name,
                                           res_item.value,
                                           res_item.namespace_uri)
                    else:
                        if value_node is None:
                            value_node = DrbQueryFuncUtil.get_value(
                                res_item)
                        else:
                            value_node = str(value_node) + \
                                         DrbQueryFuncUtil.get_string(
                                             res_item)

                else:
                    if value_node is None:
                        value_node = DrbQueryFuncUtil.get_value(res_item)
                    else:
                        value_node = str(value_node) + \
                                     DrbQueryFuncUtil.get_string(res_item)

            node._value = value_node
        return DynamicContext(node=node)

    def visitLetBinding(self, ctx: XQueryParser.LetBindingContext):
        var_namespace = None
        varname = self.visitVarName(ctx.varName())
        type_var = None

        if ctx.typeDeclaration() is not None:
            type_var = self.visitTypeDeclaration(ctx.typeDeclaration())

        for current_scope_flwor in self.flwor_scope_list:
            self.static_context.in_scope_variables = current_scope_flwor
            var_value = self.visitExprSingle(ctx.exprSingle())
            self.static_context.add_var(varname,
                                        XqueryVariable(varname,
                                                       var_namespace,
                                                       value=var_value,
                                                       type_var=type_var))

    def visitReturnClause(self, ctx: XQueryParser.ReturnClauseContext):
        if self.flwor_scope_list is None:
            return self.visitChildren(ctx)

        result = []

        for current_scope_flwor in self.flwor_scope_list:
            self.static_context.in_scope_variables = current_scope_flwor
            self.list_context = [self.dynamic_context]

            res_partial = self.visitChildren(ctx)
            DrbQueryFuncUtil.append_or_extend(res_partial, result)

        return result

    def visitForBinding(self, ctx: XQueryParser.ForBindingContext):
        if self.flwor_scope_list is None:
            return self.visitChildren(ctx)

        list_context = self.list_context
        var_name = self.visitVarName(ctx.varName())

        # TODO Type declaration
        type_var = None
        if ctx.typeDeclaration() is not None:
            type_var = self.visitTypeDeclaration(ctx.typeDeclaration())

        scope_new = []

        var_position = None
        if ctx.positionalVar() is not None:
            var_position = self.visitVarName(ctx.positionalVar().varName())

        for scope in self.flwor_scope_list:
            self.static_context.in_scope_variables = scope
            ret_partial = self.visitExprSingle(ctx.exprSingle())
            key_name = self.static_context.get_key_var(var_name)

            if isinstance(ret_partial, list):
                index = 1
                for item in ret_partial:
                    current_scope = scope.copy()
                    current_scope[key_name] = XqueryVariable(
                        var_name,
                        value=item,
                        type_var=type_var)
                    if var_position is not None:
                        current_scope[None, var_position] = XqueryVariable(
                            var_position, value=index, type_var=type_var)
                    scope_new.append(current_scope)
                    index = index + 1
            else:
                current_scope = scope.copy()
                current_scope[key_name] = XqueryVariable(
                    var_name,
                    value=ret_partial,
                    type_var=type_var)
                scope_new.append(current_scope)
            self.list_context = list_context

        self.flwor_scope_list = scope_new

        return ''

    def visitWhereClause(self, ctx: XQueryParser.WhereClauseContext):
        if self.flwor_scope_list is None:
            return self.visitChildren(ctx)

        scope_new = []

        for scope in self.flwor_scope_list:
            self.static_context.in_scope_variables = scope
            ret_partial = self.visitExprSingle(ctx.exprSingle())
            if DrbQueryFuncUtil.get_effective_boolean_value(ret_partial):
                scope_new.append(scope)
        self.flwor_scope_list = scope_new

    # Visit a parse tree produced by XQueryParser#flworExpr.
    def visitFlworExpr(self, ctx: XQueryParser.FlworExprContext):
        old_scop = self.static_context.in_scope_variables
        old_context = self.list_context
        old_scope_list = self.flwor_scope_list

        self.flwor_scope_list = [old_scop.copy()]

        res = self.visitChildren(ctx)

        self.flwor_scope_list = old_scope_list
        self.static_context.in_scope_variables = old_scop
        self.list_context = old_context
        return res

    def visitIfExpr(self, ctx: XQueryParser.IfExprContext):
        if ctx.KW_IF is None:
            return self.visitChildren(ctx)

        visitor = DrbQueryVisitor(self.dynamic_context,
                                  static_context=self.static_context,
                                  external_var_map=self.external_var_map,
                                  tokens=self.tokens)
        res = visitor.visitExpr(ctx.expr())
        if DrbQueryFuncUtil.get_effective_boolean_value(res):
            return self.visitExprSingle(ctx.thenExpr)
        else:
            return self.visitExprSingle(ctx.elseExpr)

    def visitInstanceOfExpr(self, ctx: XQueryParser.InstanceOfExprContext):
        if ctx.sequenceType() is None:
            return self.visitChildren(ctx)

        node = self.visitTreatExpr(ctx.treatExpr())
        if not isinstance(node, list):
            node = [node]

        type = self.visitSequenceType(ctx.sequenceType())
        (type_ns, type_name) = DrbQueryFuncUtil.split_namespace_name(type)
        if type_ns == 'xs' or type_ns is None:
            for item in node:
                if not DrbQueryFuncCall.xs_instance_func[type_name](
                        type_name, None, None, item):
                    return False
            return True

        return False


class DrbXqueryParserErrorListener(ErrorListener):
    def syntaxError(self, recognizer, offendingSymbol, line, column, msg, e):
        raise StaticException(ErrorXQUERY.XPST0003,
                              "ERROR: when parsing " + str(offendingSymbol) +
                              " line " + str(line) +
                              " column " + str(column) + " : " + msg)
